@@ -39,6 +39,8 @@ enum {
 	SC_CARDCTL_LIFECYCLE_GET,
 	SC_CARDCTL_LIFECYCLE_SET,
 	SC_CARDCTL_GET_SERIALNR,
+	SC_CARDCTL_GET_SE_INFO,
+	SC_CARDCTL_GET_CHV_REFERENCE_IN_SE,
 
 	/*
 	 * GPK specific calls
@@ -204,6 +206,29 @@ enum {
 	SC_CARDCTL_PIV_GENERATE_KEY,
 	SC_CARDCTL_PIV_PIN_PREFERENCE,
 	SC_CARDCTL_PIV_OBJECT_PRESENT,
+
+        /*
+	 * AuthentIC v3
+	 */
+        SC_CARDCTL_AUTHENTIC_BASE = _CTL_PREFIX('A','V','3'),
+        SC_CARDCTL_AUTHENTIC_SDO_CREATE,
+        SC_CARDCTL_AUTHENTIC_SDO_DELETE,
+        SC_CARDCTL_AUTHENTIC_SDO_STORE,
+        SC_CARDCTL_AUTHENTIC_SDO_GENERATE,
+
+        /*
+	 * IAS/ECC
+	 */
+	SC_CARDCTL_IASECC_BASE = _CTL_PREFIX('E','C','C'),
+	SC_CARDCTL_IASECC_GET_FREE_KEY_REFERENCE,
+	SC_CARDCTL_IASECC_SDO_MAGIC = _CTL_PREFIX('S','D','O') | 'M',
+	SC_CARDCTL_IASECC_SDO_MAGIC_PUT_DATA = _CTL_PREFIX('S','D','O') | 'P',
+	SC_CARDCTL_IASECC_SDO_PUT_DATA,
+	SC_CARDCTL_IASECC_SDO_KEY_RSA_PUT_DATA,
+	SC_CARDCTL_IASECC_SDO_GET_DATA,
+	SC_CARDCTL_IASECC_SDO_GENERATE,
+	SC_CARDCTL_IASECC_SDO_CREATE,
+	SC_CARDCTL_IASECC_SDO_DELETE,
 };
 
 enum {
