@@ -17,24 +17,20 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-#include "muscle.h"
-#include "internal.h"
+
+#include "config.h"
 
 #include <string.h>
 
+#include "internal.h"
+#include "muscle.h"
+
 #define MSC_RSA_PUBLIC		0x01
 #define MSC_RSA_PRIVATE 	0x02
 #define MSC_RSA_PRIVATE_CRT	0x03
 #define MSC_DSA_PUBLIC		0x04
 #define MSC_DSA_PRIVATE 	0x05
 
-#ifndef MAX
-#define MAX(x, y) (((x) > (y)) ? (x) : (y))
-#endif
-#ifndef MIN
-#define MIN(x, y) (((x) < (y)) ? (x) : (y))
-#endif
-
 static msc_id inputId = { { 0xFF, 0xFF, 0xFF, 0xFF } };
 static msc_id outputId = { { 0xFF, 0xFF, 0xFF, 0xFE } };
 
@@ -60,7 +56,7 @@ int msc_list_objects(sc_card_t* card, u8 next, mscfs_file_t* file) {
 	if(apdu.resplen == 0) /* No more left */
 		return 0;
 	if (apdu.resplen != 14) {
-		sc_error(card->ctx, "expected 14 bytes, got %d.\n", apdu.resplen);
+		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "expected 14 bytes, got %d.\n", apdu.resplen);
 		return SC_ERROR_UNKNOWN_DATA_RECEIVED;
 	}
 	memcpy(file->objectId.id, fileData, 4);
@@ -80,8 +76,8 @@ int msc_partial_read_object(sc_card_t *card, msc_id objectId, int offset, u8 *da
 	
 	sc_format_apdu(card, &apdu, SC_APDU_CASE_4_SHORT, 0x56, 0x00, 0x00);
 	
-	if (card->ctx->debug >= 2)
-		sc_debug(card->ctx, "READ: Offset: %x\tLength: %i\n", offset, dataLength);
+	sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL,
+		"READ: Offset: %x\tLength: %i\n", offset, dataLength);
 	memcpy(buffer, objectId.id, 4);
 	ulong2bebytes(buffer + 4, offset);
 	buffer[8] = (u8)dataLength;
@@ -92,23 +88,21 @@ int msc_partial_read_object(sc_card_t *card, msc_id objectId, int offset, u8 *da
 	apdu.resplen = dataLength;
 	apdu.resp = data; 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	if(apdu.sw1 == 0x90 && apdu.sw2 == 0x00)
 		return dataLength;
 	if(apdu.sw1 == 0x9C) {
 		if(apdu.sw2 == 0x07) {
-			SC_FUNC_RETURN(card->ctx, 2, SC_ERROR_FILE_NOT_FOUND);
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_ERROR_FILE_NOT_FOUND);
 		} else if(apdu.sw2 == 0x06) {
-			SC_FUNC_RETURN(card->ctx, 2, SC_ERROR_NOT_ALLOWED);
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_ERROR_NOT_ALLOWED);
 		} else if(apdu.sw2 == 0x0F) {
 			/* GUESSED */
-			SC_FUNC_RETURN(card->ctx, 2, SC_ERROR_INVALID_ARGUMENTS);
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_ERROR_INVALID_ARGUMENTS);
 		}
 	}
-	if (card->ctx->debug >= 2) {
-		sc_debug(card->ctx, "got strange SWs: 0x%02X 0x%02X\n",
-		     apdu.sw1, apdu.sw2);
-	}
+	sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL,
+		"got strange SWs: 0x%02X 0x%02X\n", apdu.sw1, apdu.sw2);
 	return dataLength;
 	
 }
@@ -121,7 +115,7 @@ int msc_read_object(sc_card_t *card, msc_id objectId, int offset, u8 *data, size
 
 	for(i = 0; i < dataLength; i += max_read_unit) {
 		r = msc_partial_read_object(card, objectId, offset + i, data + i, MIN(dataLength - i, max_read_unit));
-		SC_TEST_RET(card->ctx, r, "Error in partial object read");
+		SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Error in partial object read");
 	}
 	return dataLength;
 }
@@ -135,16 +129,15 @@ int msc_zero_object(sc_card_t *card, msc_id objectId, size_t dataLength)
 	memset(zeroBuffer, 0, max_write_unit);
 	for(i = 0; i < dataLength; i += max_write_unit) {
 		int r = msc_partial_update_object(card, objectId, i, zeroBuffer, MIN(dataLength - i, max_write_unit));
-		SC_TEST_RET(card->ctx, r, "Error in zeroing file update");
+		SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Error in zeroing file update");
 	}
 	return 0;
 }
 
-int msc_create_object(sc_card_t *card, msc_id objectId, size_t objectSize, unsigned short read, unsigned short write, unsigned short deletion)
+int msc_create_object(sc_card_t *card, msc_id objectId, size_t objectSize, unsigned short readAcl, unsigned short writeAcl, unsigned short deleteAcl)
 {
 	u8 buffer[14];
 	sc_apdu_t apdu;
-	unsigned short readAcl = read, writeAcl = write, deleteAcl = deletion;
 	int r;
 
 	sc_format_apdu(card, &apdu, SC_APDU_CASE_3_SHORT, 0x5A, 0x00, 0x00);
@@ -158,20 +151,20 @@ int msc_create_object(sc_card_t *card, msc_id objectId, size_t objectSize, unsig
 	ushort2bebytes(buffer + 10, writeAcl);
 	ushort2bebytes(buffer + 12, deleteAcl);
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	if(apdu.sw1 == 0x90 && apdu.sw2 == 0x00)
 		return objectSize;
 	if(apdu.sw1 == 0x9C) {
 		if(apdu.sw2 == 0x01) {
-			SC_FUNC_RETURN(card->ctx, 2, SC_ERROR_MEMORY_FAILURE);
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_ERROR_MEMORY_FAILURE);
 		} else if(apdu.sw2 == 0x08) {
-			SC_FUNC_RETURN(card->ctx, 2, SC_ERROR_FILE_ALREADY_EXISTS);
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_ERROR_FILE_ALREADY_EXISTS);
 		} else if(apdu.sw2 == 0x06) {
-			SC_FUNC_RETURN(card->ctx, 2, SC_ERROR_NOT_ALLOWED);
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_ERROR_NOT_ALLOWED);
 		}
 	}
 	if (card->ctx->debug >= 2) {
-		sc_debug(card->ctx, "got strange SWs: 0x%02X 0x%02X\n",
+		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "got strange SWs: 0x%02X 0x%02X\n",
 		     apdu.sw1, apdu.sw2);
 	}
 	msc_zero_object(card, objectId, objectSize);
@@ -188,7 +181,7 @@ int msc_partial_update_object(sc_card_t *card, msc_id objectId, int offset, cons
 	sc_format_apdu(card, &apdu, SC_APDU_CASE_3_SHORT, 0x54, 0x00, 0x00);
 	apdu.lc = dataLength + 9;
 	if (card->ctx->debug >= 2)
-		sc_debug(card->ctx, "WRITE: Offset: %x\tLength: %i\n", offset, dataLength);
+		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "WRITE: Offset: %x\tLength: %i\n", offset, dataLength);
 	
 	memcpy(buffer, objectId.id, 4);
 	ulong2bebytes(buffer + 4, offset);
@@ -197,21 +190,21 @@ int msc_partial_update_object(sc_card_t *card, msc_id objectId, int offset, cons
 	apdu.data = buffer;
 	apdu.datalen = apdu.lc;
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	if(apdu.sw1 == 0x90 && apdu.sw2 == 0x00)
 		return dataLength;
 	if(apdu.sw1 == 0x9C) {
 		if(apdu.sw2 == 0x07) {
-			SC_FUNC_RETURN(card->ctx, 2, SC_ERROR_FILE_NOT_FOUND);
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_ERROR_FILE_NOT_FOUND);
 		} else if(apdu.sw2 == 0x06) {
-			SC_FUNC_RETURN(card->ctx, 2, SC_ERROR_NOT_ALLOWED);
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_ERROR_NOT_ALLOWED);
 		} else if(apdu.sw2 == 0x0F) {
 			/* GUESSED */
-			SC_FUNC_RETURN(card->ctx, 2, SC_ERROR_INVALID_ARGUMENTS);
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_ERROR_INVALID_ARGUMENTS);
 		}
 	}
 	if (card->ctx->debug >= 2) {
-		sc_debug(card->ctx, "got strange SWs: 0x%02X 0x%02X\n",
+		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "got strange SWs: 0x%02X 0x%02X\n",
 		     apdu.sw1, apdu.sw2);
 	}
 	return dataLength;
@@ -224,7 +217,7 @@ int msc_update_object(sc_card_t *card, msc_id objectId, int offset, const u8 *da
 	size_t max_write_unit = MSC_MAX_SEND - 9;
 	for(i = 0; i < dataLength; i += max_write_unit) {
 		r = msc_partial_update_object(card, objectId, offset + i, data + i, MIN(dataLength - i, max_write_unit));
-		SC_TEST_RET(card->ctx, r, "Error in partial object update");
+		SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "Error in partial object update");
 	}
 	return dataLength;
 }
@@ -239,18 +232,18 @@ int msc_delete_object(sc_card_t *card, msc_id objectId, int zero)
 	apdu.data = objectId.id;
 	apdu.datalen = 4;
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	if(apdu.sw1 == 0x90 && apdu.sw2 == 0x00)
 		return 0;
 	if(apdu.sw1 == 0x9C) {
 		if(apdu.sw2 == 0x07) {
-			SC_FUNC_RETURN(card->ctx, 2, SC_ERROR_FILE_NOT_FOUND);
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_ERROR_FILE_NOT_FOUND);
 		} else if(apdu.sw2 == 0x06) {
-			SC_FUNC_RETURN(card->ctx, 2, SC_ERROR_NOT_ALLOWED);
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_ERROR_NOT_ALLOWED);
 		}
 	}
 	if (card->ctx->debug >= 2) {
-		sc_debug(card->ctx, "got strange SWs: 0x%02X 0x%02X\n",
+		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "got strange SWs: 0x%02X 0x%02X\n",
 		     apdu.sw1, apdu.sw2);
 	}
 	return 0;
@@ -269,11 +262,11 @@ int msc_select_applet(sc_card_t *card, u8 *appletId, size_t appletIdLength)
 	apdu.le = 0;
 	
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	if(apdu.sw1 == 0x90 && apdu.sw2 == 0x00)
 		return 1;
 	
-	SC_FUNC_RETURN(card->ctx, 2,  SC_ERROR_CARD_CMD_FAILED);
+	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE,  SC_ERROR_CARD_CMD_FAILED);
 }
 
 /* Truncate the nulls at the end of a PIN, useful in padding is unnecessarily added */
@@ -296,20 +289,20 @@ int msc_verify_pin(sc_card_t *card, int pinNumber, const u8 *pinValue, int pinLe
 	if(tries)
 		*tries = -1;
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	if(apdu.sw1 == 0x90 && apdu.sw2 == 0x00) {
 		return 0;
 	} else if(apdu.sw1 == 0x63) { /* Invalid auth */
 		if(tries)
 			*tries = apdu.sw2 & 0x0F;
-		SC_FUNC_RETURN(card->ctx, 0,  SC_ERROR_PIN_CODE_INCORRECT);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL,  SC_ERROR_PIN_CODE_INCORRECT);
 	} else if(apdu.sw1 == 0x9C && apdu.sw2 == 0x02) {
-		SC_FUNC_RETURN(card->ctx, 0,  SC_ERROR_PIN_CODE_INCORRECT);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL,  SC_ERROR_PIN_CODE_INCORRECT);
 	} else if(apdu.sw1 == 0x69 && apdu.sw2 == 0x83) {
-		SC_FUNC_RETURN(card->ctx, 0, SC_ERROR_AUTH_METHOD_BLOCKED);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, SC_ERROR_AUTH_METHOD_BLOCKED);
 	}
 	
-	SC_FUNC_RETURN(card->ctx, 2,  SC_ERROR_PIN_CODE_INCORRECT);
+	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE,  SC_ERROR_PIN_CODE_INCORRECT);
 }
 
 /* USE ISO_VERIFY due to tries return */
@@ -341,20 +334,20 @@ int msc_unblock_pin(sc_card_t *card, int pinNumber, const u8 *pukValue, int pukL
 	if(tries)
 		*tries = -1;
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	if(apdu.sw1 == 0x90 && apdu.sw2 == 0x00) {
 		return 0;
 	} else if(apdu.sw1 == 0x63) { /* Invalid auth */
 		if(tries)
 			*tries = apdu.sw2 & 0x0F;
-		SC_FUNC_RETURN(card->ctx, 0,  SC_ERROR_PIN_CODE_INCORRECT);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL,  SC_ERROR_PIN_CODE_INCORRECT);
 	} else if(apdu.sw1 == 0x9C && apdu.sw2 == 0x02) {
-		SC_FUNC_RETURN(card->ctx, 0,  SC_ERROR_PIN_CODE_INCORRECT);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL,  SC_ERROR_PIN_CODE_INCORRECT);
 	} else if(apdu.sw1 == 0x69 && apdu.sw2 == 0x83) {
-		SC_FUNC_RETURN(card->ctx, 0, SC_ERROR_AUTH_METHOD_BLOCKED);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, SC_ERROR_AUTH_METHOD_BLOCKED);
 	}
 	
-	SC_FUNC_RETURN(card->ctx, 2,  SC_ERROR_PIN_CODE_INCORRECT);
+	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE,  SC_ERROR_PIN_CODE_INCORRECT);
 }
 
 void msc_unblock_pin_apdu(sc_card_t *card, sc_apdu_t *apdu, u8* buffer, size_t bufferLength, int pinNumber, const u8 *pukValue, int pukLength)
@@ -383,20 +376,20 @@ int msc_change_pin(sc_card_t *card, int pinNumber, const u8 *pinValue, int pinLe
 	if(tries)
 		*tries = -1;
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	if(apdu.sw1 == 0x90 && apdu.sw2 == 0x00) {
 		return 0;
 	} else if(apdu.sw1 == 0x63) { /* Invalid auth */
 		if(tries)
 			*tries = apdu.sw2 & 0x0F;
-		SC_FUNC_RETURN(card->ctx, 0,  SC_ERROR_PIN_CODE_INCORRECT);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL,  SC_ERROR_PIN_CODE_INCORRECT);
 	} else if(apdu.sw1 == 0x9C && apdu.sw2 == 0x02) {
-		SC_FUNC_RETURN(card->ctx, 0,  SC_ERROR_PIN_CODE_INCORRECT);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL,  SC_ERROR_PIN_CODE_INCORRECT);
 	} else if(apdu.sw1 == 0x69 && apdu.sw2 == 0x83) {
-		SC_FUNC_RETURN(card->ctx, 0, SC_ERROR_AUTH_METHOD_BLOCKED);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, SC_ERROR_AUTH_METHOD_BLOCKED);
 	}
 	
-	SC_FUNC_RETURN(card->ctx, 2,  SC_ERROR_PIN_CODE_INCORRECT);
+	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE,  SC_ERROR_PIN_CODE_INCORRECT);
 }
 
 /* USE ISO_VERIFY due to tries return */
@@ -421,7 +414,6 @@ void msc_change_pin_apdu(sc_card_t *card, sc_apdu_t *apdu, u8* buffer, size_t bu
 	*ptr = newPinLength;
 	ptr++;
 	memcpy(ptr, newPin, newPinLength);
-	ptr += newPinLength;
 	apdu->lc = pinLength + newPinLength + 2;
 	apdu->datalen = apdu->lc;
 	apdu->data = buffer;
@@ -442,7 +434,7 @@ int msc_get_challenge(sc_card_t *card, unsigned short dataLength, unsigned short
 	assert(dataLength < MSC_MAX_READ - 9); /* Output buffer doesn't seem to operate as desired.... nobody can read/delete */
 	
 	buffer = malloc(len);
-	if(!buffer) SC_FUNC_RETURN(card->ctx, 0, SC_ERROR_OUT_OF_MEMORY);
+	if(!buffer) SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, SC_ERROR_OUT_OF_MEMORY);
 	ptr = buffer;
 	ushort2bebytes(ptr, dataLength);
 	ptr+=2;
@@ -458,7 +450,7 @@ int msc_get_challenge(sc_card_t *card, unsigned short dataLength, unsigned short
 	
 	if(location == 1) {
 		u8* outputBuffer = malloc(dataLength + 2);
-		if(outputBuffer == NULL) SC_FUNC_RETURN(card->ctx, 0, SC_ERROR_OUT_OF_MEMORY);
+		if(outputBuffer == NULL) SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, SC_ERROR_OUT_OF_MEMORY);
 		apdu.le = dataLength + 2;
 		apdu.resp = outputBuffer;
 		apdu.resplen = dataLength + 2;
@@ -469,7 +461,7 @@ int msc_get_challenge(sc_card_t *card, unsigned short dataLength, unsigned short
 		free(apdu.resp);
 	}
 	free(buffer);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	if(location == 1) {
 		if(apdu.sw1 == 0x90 && apdu.sw2 == 0x00) {
 			return SC_SUCCESS;
@@ -477,32 +469,30 @@ int msc_get_challenge(sc_card_t *card, unsigned short dataLength, unsigned short
 			r = sc_check_sw(card, apdu.sw1, apdu.sw2);
 			if (r) {
 				if (card->ctx->debug >= 2) {
-					sc_debug(card->ctx, "got strange SWs: 0x%02X 0x%02X\n",
+					sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "got strange SWs: 0x%02X 0x%02X\n",
 					     apdu.sw1, apdu.sw2);
 				}
-				SC_FUNC_RETURN(card->ctx, 0, r);
+				SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
 			}
-			SC_FUNC_RETURN(card->ctx, 0, SC_ERROR_CARD_CMD_FAILED);
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, SC_ERROR_CARD_CMD_FAILED);
 		}
 	} else {
 		if(apdu.sw1 != 0x90 || apdu.sw2 != 0x00) {
 			r = sc_check_sw(card, apdu.sw1, apdu.sw2);
 			if (r) {
 				if (card->ctx->debug >= 2) {
-					sc_debug(card->ctx, "got strange SWs: 0x%02X 0x%02X\n",
+					sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "got strange SWs: 0x%02X 0x%02X\n",
 					     apdu.sw1, apdu.sw2);
 				}
-				SC_FUNC_RETURN(card->ctx, 0, r);
+				SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
 			}
-			SC_FUNC_RETURN(card->ctx, 0, SC_ERROR_CARD_CMD_FAILED);
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, SC_ERROR_CARD_CMD_FAILED);
 		}
 		r = msc_read_object(card, inputId, 2, outputData, dataLength);
 		if(r < 0)
-			SC_FUNC_RETURN(card->ctx, 0, r);
-		sc_ctx_suppress_errors_on(card->ctx);
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
 		msc_delete_object(card, inputId,0);
-		sc_ctx_suppress_errors_off(card->ctx);
-		SC_FUNC_RETURN(card->ctx, 0, r);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
 	}
 }
 
@@ -545,19 +535,19 @@ int msc_generate_keypair(sc_card_t *card, int privateKey, int publicKey, int alg
 	apdu.lc = 16;
 	
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	if(apdu.sw1 == 0x90 && apdu.sw2 == 0x00) {
 		return 0;
 	}
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
 	if (r) {
 		if (card->ctx->debug >= 2) {
-			sc_debug(card->ctx, "got strange SWs: 0x%02X 0x%02X\n",
+			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "got strange SWs: 0x%02X 0x%02X\n",
 			     apdu.sw1, apdu.sw2);
 		}
-		SC_FUNC_RETURN(card->ctx, 0, r);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
 	}
-	SC_FUNC_RETURN(card->ctx, 0, SC_ERROR_CARD_CMD_FAILED);
+	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, SC_ERROR_CARD_CMD_FAILED);
 }
 
 int msc_extract_key(sc_card_t *card, 
@@ -572,19 +562,19 @@ int msc_extract_key(sc_card_t *card,
 	apdu.datalen = 1;
 	apdu.lc = 1;
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	if(apdu.sw1 == 0x90 && apdu.sw2 == 0x00) {
 		return 0;
 	}
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
 	if (r) {
 		if (card->ctx->debug >= 2) {
-			sc_debug(card->ctx, "got strange SWs: 0x%02X 0x%02X\n",
+			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "got strange SWs: 0x%02X 0x%02X\n",
 			     apdu.sw1, apdu.sw2);
 		}
-		SC_FUNC_RETURN(card->ctx, 0, r);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
 	}
-	SC_FUNC_RETURN(card->ctx, 0, SC_ERROR_CARD_CMD_FAILED);
+	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, SC_ERROR_CARD_CMD_FAILED);
 }
 
 int msc_extract_rsa_public_key(sc_card_t *card, 
@@ -601,36 +591,36 @@ int msc_extract_rsa_public_key(sc_card_t *card,
 
 	assert(modLength && expLength && modulus && exponent);
 	r = msc_extract_key(card, keyLocation);
-	if(r < 0) SC_FUNC_RETURN(card->ctx, 0, r);
+	if(r < 0) SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
 	
 	/* Read keyType, keySize, and what should be the modulus size */
 	r = msc_read_object(card, inputId, fileLocation, buffer, 5);
 	fileLocation += 5;
-	if(r < 0) SC_FUNC_RETURN(card->ctx, 0, r);
+	if(r < 0) SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
 	
-	if(buffer[0] != MSC_RSA_PUBLIC) SC_FUNC_RETURN(card->ctx, 0, SC_ERROR_UNKNOWN_DATA_RECEIVED);
+	if(buffer[0] != MSC_RSA_PUBLIC) SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, SC_ERROR_UNKNOWN_DATA_RECEIVED);
 	*modLength = (buffer[3] << 8) | buffer[4];
 	/* Read the modulus and the exponent length */
 	assert(*modLength + 2 < buffer_size);
 	
 	r = msc_read_object(card, inputId, fileLocation, buffer, *modLength + 2);
 	fileLocation += *modLength + 2;
-	if(r < 0) SC_FUNC_RETURN(card->ctx, 0, r);
+	if(r < 0) SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
 	
 	*modulus = malloc(*modLength);
-	if(!*modulus) SC_FUNC_RETURN(card->ctx, 0, SC_ERROR_OUT_OF_MEMORY);
+	if(!*modulus) SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, SC_ERROR_OUT_OF_MEMORY);
 	memcpy(*modulus, buffer, *modLength);
 	*expLength = (buffer[*modLength] << 8) | buffer[*modLength + 1];
 	assert(*expLength < buffer_size);
 	r = msc_read_object(card, inputId, fileLocation, buffer, *expLength);
 	if(r < 0) {
 		free(*modulus); *modulus = NULL;
-		SC_FUNC_RETURN(card->ctx, 0, r);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
 	}
 	*exponent = malloc(*expLength);
 	if(!*exponent) {
 		free(*modulus);
-		SC_FUNC_RETURN(card->ctx, 0, SC_ERROR_OUT_OF_MEMORY);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, SC_ERROR_OUT_OF_MEMORY);
 	}
 	memcpy(*exponent, buffer, *expLength);
 	return 0;
@@ -673,7 +663,7 @@ int msc_compute_crypt_init(sc_card_t *card,
 	memcpy(ptr, initData, dataLength);
 	
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	if(apdu.sw1 == 0x90 && apdu.sw2 == 0x00) {
 		short receivedData = outputBuffer[0] << 8 | outputBuffer[1];
 		 *outputDataLength = receivedData;
@@ -685,12 +675,12 @@ int msc_compute_crypt_init(sc_card_t *card,
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
 	if (r) {
 		if (card->ctx->debug >= 2) {
-			sc_debug(card->ctx, "init: got strange SWs: 0x%02X 0x%02X\n",
+			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "init: got strange SWs: 0x%02X 0x%02X\n",
 			     apdu.sw1, apdu.sw2);
 		}
-		SC_FUNC_RETURN(card->ctx, 0, r);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
 	}
-	SC_FUNC_RETURN(card->ctx, 0, SC_ERROR_CARD_CMD_FAILED);
+	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, SC_ERROR_CARD_CMD_FAILED);
 }
 
 #if 0
@@ -729,7 +719,7 @@ int msc_compute_crypt_process(
 	memcpy(ptr, inputData, dataLength);
 	
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	if(apdu.sw1 == 0x90 && apdu.sw2 == 0x00) {
 		short receivedData = outputBuffer[0] << 8 | outputBuffer[1];
 		 *outputDataLength = receivedData;
@@ -741,12 +731,12 @@ int msc_compute_crypt_process(
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
 	if (r) {
 		if (card->ctx->debug >= 2) {
-			sc_debug(card->ctx, "process: got strange SWs: 0x%02X 0x%02X\n",
+			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "process: got strange SWs: 0x%02X 0x%02X\n",
 			     apdu.sw1, apdu.sw2);
 		}
-		SC_FUNC_RETURN(card->ctx, 0, r);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
 	}
-	SC_FUNC_RETURN(card->ctx, 0, SC_ERROR_CARD_CMD_FAILED);
+	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, SC_ERROR_CARD_CMD_FAILED);
 }
 #endif
 
@@ -781,7 +771,7 @@ int msc_compute_crypt_final(
 	memcpy(ptr, inputData, dataLength);
 	
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	if(apdu.sw1 == 0x90 && apdu.sw2 == 0x00) {
 		short receivedData = outputBuffer[0] << 8 | outputBuffer[1];
 		*outputDataLength = receivedData;
@@ -792,12 +782,12 @@ int msc_compute_crypt_final(
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
 	if (r) {
 		if (card->ctx->debug >= 2) {
-			sc_debug(card->ctx, "final: got strange SWs: 0x%02X 0x%02X\n",
+			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "final: got strange SWs: 0x%02X 0x%02X\n",
 			     apdu.sw1, apdu.sw2);
 		}
-		SC_FUNC_RETURN(card->ctx, 0, r);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
 	}
-	SC_FUNC_RETURN(card->ctx, 0, SC_ERROR_CARD_CMD_FAILED);
+	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, SC_ERROR_CARD_CMD_FAILED);
 }
 
 /* Stream data to the card through file IO */
@@ -829,29 +819,25 @@ static int msc_compute_crypt_final_object(
 	ptr++;
 	memcpy(ptr, inputData, dataLength);
 
-	sc_ctx_suppress_errors_on(card->ctx);
 	r = msc_create_object(card, outputId, dataLength + 2, 0x02, 0x02, 0x02);
 	if(r < 0) { 
 		if(r == SC_ERROR_FILE_ALREADY_EXISTS) {
 			r = msc_delete_object(card, outputId, 0);
 			if(r < 0) {
-				sc_ctx_suppress_errors_off(card->ctx);
-				SC_FUNC_RETURN(card->ctx, 2, r);
+				SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, r);
 			}
 			r = msc_create_object(card, outputId, dataLength + 2, 0x02, 0x02, 0x02);
 			if(r < 0) {
-				sc_ctx_suppress_errors_off(card->ctx);
-				SC_FUNC_RETURN(card->ctx, 2, r);
+				SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, r);
 			}
 		}
 	}
-	sc_ctx_suppress_errors_off(card->ctx);
 
 	r = msc_update_object(card, outputId, 0, buffer + 1, dataLength + 2);
 	if(r < 0) return r; 
 	
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	if(apdu.sw1 == 0x90 && apdu.sw2 == 0x00) {
 		r = msc_read_object(card, inputId, 2, outputData, dataLength);
 		*outputDataLength = dataLength;
@@ -862,18 +848,16 @@ static int msc_compute_crypt_final_object(
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
 	if (r) {
 		if (card->ctx->debug >= 2) {
-			sc_debug(card->ctx, "final: got strange SWs: 0x%02X 0x%02X\n",
+			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "final: got strange SWs: 0x%02X 0x%02X\n",
 			     apdu.sw1, apdu.sw2);
 		}
 	} else {
 		r = SC_ERROR_CARD_CMD_FAILED;
 	}
-	/* no error checks.. this is last ditch cleanup */	
-	sc_ctx_suppress_errors_on(card->ctx);
+	/* this is last ditch cleanup */	
 	msc_delete_object(card, outputId, 0);
-	sc_ctx_suppress_errors_off(card->ctx);
 	
-	SC_FUNC_RETURN(card->ctx, 0, r);
+	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
 }
 
 int msc_compute_crypt(sc_card_t *card, 
@@ -904,7 +888,7 @@ int msc_compute_crypt(sc_card_t *card,
 		outPtr, 
 		toSend,
 		&received);
-	if(r < 0) SC_FUNC_RETURN(card->ctx, 0, r);
+	if(r < 0) SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
 	left -= toSend;
 	inPtr += toSend;
 	outPtr += received;
@@ -919,7 +903,7 @@ int msc_compute_crypt(sc_card_t *card,
 			outPtr,
 			toSend,
 			&received);
-		if(r < 0) SC_FUNC_RETURN(card->ctx, 0, r);
+		if(r < 0) SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
 	} else { /* Data is too big: use objects */
 		r = msc_compute_crypt_final_object(card,
 			keyLocation,
@@ -927,10 +911,8 @@ int msc_compute_crypt(sc_card_t *card,
 			outPtr,
 			toSend,
 			&received);
-		if(r < 0) SC_FUNC_RETURN(card->ctx, 0, r);
+		if(r < 0) SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
 	}	
-	left -= toSend;
-	inPtr += toSend;
 	outPtr += received;
 
 	return outPtr - outputData; /* Amt received */
@@ -945,8 +927,8 @@ int msc_import_key(sc_card_t *card,
 	int keyLocation,
 	sc_cardctl_muscle_key_info_t *data)
 {
-	unsigned short read = 0xFFFF,
-		write = 0x0002,
+	unsigned short readAcl = 0xFFFF,
+		writeAcl = 0x0002,
 		use = 0x0002,
 		keySize = data->keySize;
 	int bufferSize = 0;
@@ -959,16 +941,16 @@ int msc_import_key(sc_card_t *card,
 	if(data->keyType == 0x02) {
 		if( (data->pLength == 0 || !data->pValue)
 		|| (data->modLength == 0 || !data->modValue))
-			SC_FUNC_RETURN(card->ctx, 2, SC_ERROR_INVALID_ARGUMENTS); 
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_ERROR_INVALID_ARGUMENTS); 
 	} else if(data->keyType == 0x03) {
 		if( (data->pLength == 0 || !data->pValue)
 		|| (data->qLength == 0 || !data->qValue)
 		|| (data->pqLength == 0 || !data->pqValue)
 		|| (data->dp1Length == 0 || !data->dp1Value)
 		|| (data->dq1Length == 0 || !data->dq1Value))
-			SC_FUNC_RETURN(card->ctx, 2, SC_ERROR_INVALID_ARGUMENTS); 
+			SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_ERROR_INVALID_ARGUMENTS); 
 	} else {
-		SC_FUNC_RETURN(card->ctx, 2, SC_ERROR_INVALID_ARGUMENTS);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, SC_ERROR_INVALID_ARGUMENTS);
 	}
 	
 	if(data->keyType == 0x02) {
@@ -979,7 +961,7 @@ int msc_import_key(sc_card_t *card,
 			+ data->dp1Length + data->dq1Length;
 	}
 	buffer = malloc(bufferSize);
-	if(!buffer) SC_FUNC_RETURN(card->ctx, 0, SC_ERROR_OUT_OF_MEMORY);
+	if(!buffer) SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, SC_ERROR_OUT_OF_MEMORY);
 	p = buffer;
 	*p = 0x00; p++; /* Encoding plain */
 	*p = data->keyType; p++; /* RSA_PRIVATE */
@@ -996,25 +978,21 @@ int msc_import_key(sc_card_t *card,
 		CPYVAL(dq1);
 	}
 	
-	sc_ctx_suppress_errors_on(card->ctx);
 	r = msc_create_object(card, outputId, bufferSize, 0x02, 0x02, 0x02);
 	if(r < 0) { 
 		if(r == SC_ERROR_FILE_ALREADY_EXISTS) {
 			r = msc_delete_object(card, outputId, 0);
 			if(r < 0) {
-				sc_ctx_suppress_errors_off(card->ctx);
 				free(buffer);
-				SC_FUNC_RETURN(card->ctx, 2, r);
+				SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, r);
 			}
 			r = msc_create_object(card, outputId, bufferSize, 0x02, 0x02, 0x02);
 			if(r < 0) {
-				sc_ctx_suppress_errors_off(card->ctx);
 				free(buffer);
-				SC_FUNC_RETURN(card->ctx, 2, r);
+				SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_VERBOSE, r);
 			}
 		}
 	}
-	sc_ctx_suppress_errors_off(card->ctx);
 	
 	r = msc_update_object(card, outputId, 0, buffer, bufferSize);
 	free(buffer);
@@ -1026,11 +1004,11 @@ int msc_import_key(sc_card_t *card,
 	apdu.data = apduBuffer;
 	apdu.datalen = 6;
 	p = apduBuffer;
-	ushort2bebytes(p, read); p+=2;
-	ushort2bebytes(p, write); p+=2;
-	ushort2bebytes(p, use); p+=2;	
+	ushort2bebytes(p, readAcl); p+=2;
+	ushort2bebytes(p, writeAcl); p+=2;
+	ushort2bebytes(p, use); 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	if(apdu.sw1 == 0x90 && apdu.sw2 == 0x00) {
 		msc_delete_object(card, outputId, 0);
 		return 0;
@@ -1038,36 +1016,16 @@ int msc_import_key(sc_card_t *card,
 	r = sc_check_sw(card, apdu.sw1, apdu.sw2);
 	if (r) {
 		if (card->ctx->debug >= 2) {
-			sc_debug(card->ctx, "keyimport: got strange SWs: 0x%02X 0x%02X\n",
+			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "keyimport: got strange SWs: 0x%02X 0x%02X\n",
 			     apdu.sw1, apdu.sw2);
 		}
-		/* no error checks.. this is last ditch cleanup */
-		sc_ctx_suppress_errors_on(card->ctx);
+		/* this is last ditch cleanup */
 		msc_delete_object(card, outputId, 0);
-		sc_ctx_suppress_errors_off(card->ctx);
-		SC_FUNC_RETURN(card->ctx, 0, r);
+		SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, r);
 	}
-	/* no error checks.. this is last ditch cleanup */
-	sc_ctx_suppress_errors_on(card->ctx);
+	/* this is last ditch cleanup */
 	msc_delete_object(card, outputId, 0);
-	sc_ctx_suppress_errors_off(card->ctx);
 
-	SC_FUNC_RETURN(card->ctx, 0, SC_ERROR_CARD_CMD_FAILED);
+	SC_FUNC_RETURN(card->ctx, SC_LOG_DEBUG_NORMAL, SC_ERROR_CARD_CMD_FAILED);
 }
 #undef CPYVAL
-
-/* For future implementation of check_sw */
-/*
-switch(apdu.sw1) {
-	case 0x9C:
-	switch(apdu.sw2) {
-		case 0x03: // Operation not allowed
-		case 0x05: // Unsupported
-		case 0x06: // Unauthorized
-		case 0x11: // Bad private key num
-		case 0x12: // Bad public key num
-		case 0x0E:
-		case 0x0F: // Invalid parameters...
-	}
-}
-*/
