@@ -349,8 +349,6 @@ sc_oberthur_parse_tokeninfo (struct sc_pkcs15_card *p15card,
 		SC_TEST_RET(ctx, SC_LOG_DEBUG_NORMAL, SC_ERROR_INVALID_ARGUMENTS, "Cannot parse token info");
 
 	memset(label, 0, sizeof(label));
-	if (len > sizeof(label) - 1)
-		len = sizeof(label) - 1;
 
 	memcpy(label, buff, 0x20);
 	ii = 0x20;
@@ -504,7 +502,7 @@ sc_oberthur_parse_privateinfo (struct sc_pkcs15_card *p15card,
 
 				sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "postpone adding of the private keys");
 				sc_format_path("5011A5A5", &path);
-				rv = sc_pkcs15_add_df(p15card, SC_PKCS15_PRKDF, &path, NULL);
+				rv = sc_pkcs15_add_df(p15card, SC_PKCS15_PRKDF, &path);
 				SC_TEST_RET(ctx, SC_LOG_DEBUG_NORMAL, rv, "Add PrkDF error");
 				no_more_private_keys = 1;
 			}
@@ -523,7 +521,7 @@ sc_oberthur_parse_privateinfo (struct sc_pkcs15_card *p15card,
 
 				sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "postpone adding of the private data");
 				sc_format_path("5011A6A6", &path);
-				rv = sc_pkcs15_add_df(p15card, SC_PKCS15_DODF, &path, NULL);
+				rv = sc_pkcs15_add_df(p15card, SC_PKCS15_DODF, &path);
 				SC_TEST_RET(ctx, SC_LOG_DEBUG_NORMAL, rv, "Add DODF error");
 				no_more_private_data = 1;
 			}
@@ -555,7 +553,7 @@ sc_pkcs15emu_oberthur_add_pubkey(struct sc_pkcs15_card *p15card,
 	struct sc_pkcs15_object key_obj;
 	char ch_tmp[0x100];
 	unsigned char *info_blob;
-	size_t len, info_len, offs, sz;
+	size_t len, info_len, offs;
 	unsigned flags;
 	int rv;
 
@@ -584,7 +582,8 @@ sc_pkcs15emu_oberthur_add_pubkey(struct sc_pkcs15_card *p15card,
 		SC_TEST_RET(ctx, SC_LOG_DEBUG_NORMAL, SC_ERROR_UNKNOWN_DATA_RECEIVED, "Failed to add public key: no 'Label'");
 	len = *(info_blob + offs + 1) + *(info_blob + offs) * 0x100;
 	if (len)   {
-		sz = len > sizeof(key_obj.label) - 1 ? sizeof(key_obj.label) - 1 : len;
+		if (len > sizeof(key_obj.label) - 1)
+			len = sizeof(key_obj.label) - 1;
 		memcpy(key_obj.label, info_blob + offs + 2, len);
 	}
 	offs += 2 + len;
@@ -628,7 +627,7 @@ sc_pkcs15emu_oberthur_add_cert(struct sc_pkcs15_card *p15card, unsigned int file
 	struct sc_pkcs15_cert_info cinfo;
 	struct sc_pkcs15_object cobj;
 	unsigned char *info_blob, *cert_blob;
-	size_t info_len, cert_len, len, offs, sz;
+	size_t info_len, cert_len, len, offs;
 	unsigned flags;
 	int rv;
 	char ch_tmp[0x20];
@@ -653,7 +652,8 @@ sc_pkcs15emu_oberthur_add_cert(struct sc_pkcs15_card *p15card, unsigned int file
 		SC_TEST_RET(ctx, SC_LOG_DEBUG_NORMAL, SC_ERROR_UNKNOWN_DATA_RECEIVED, "Failed to add certificate: no 'CN'");
 	len = *(info_blob + offs + 1) + *(info_blob + offs) * 0x100;
 	if (len)   {
-		sz = len > sizeof(cobj.label) - 1 ? sizeof(cobj.label) - 1 : len;
+		if (len > sizeof(cobj.label) - 1)
+			len = sizeof(cobj.label) - 1;
 		memcpy(cobj.label, info_blob + offs + 2, len);
 	}
 	offs += 2 + len;
@@ -666,7 +666,6 @@ sc_pkcs15emu_oberthur_add_cert(struct sc_pkcs15_card *p15card, unsigned int file
 		SC_TEST_RET(ctx, SC_LOG_DEBUG_NORMAL, SC_ERROR_INVALID_DATA, "Failed to add certificate: invalie 'ID' length");
 	memcpy(cinfo.id.value, info_blob + offs + 2, len);
 	cinfo.id.len = len;
-	offs += 2 + len;
 
 	/* Ignore subject, issuer and serial */
 
