@@ -20,13 +20,16 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include "internal.h"
-#include <opensc/pkcs15.h>
-#include <opensc/log.h>
+#include "config.h"
+
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
-#include <compat_strlcpy.h>
+
+#include "common/compat_strlcpy.h"
+#include "internal.h"
+#include "pkcs15.h"
+#include "log.h"
 
 int sc_pkcs15emu_postecert_init_ex(sc_pkcs15_card_t *, sc_pkcs15emu_opt_t *);
 
@@ -190,9 +193,9 @@ static int sc_pkcs15emu_postecert_init(sc_pkcs15_card_t * p15card)
 		goto failed;
 	}
 
-	set_string(&p15card->label, "Postecert & Cnipa Card");
-	set_string(&p15card->manufacturer_id, "Postecert");
-	set_string(&p15card->serial_number, "0000");
+	set_string(&p15card->tokeninfo->label, "Postecert & Cnipa Card");
+	set_string(&p15card->tokeninfo->manufacturer_id, "Postecert");
+	set_string(&p15card->tokeninfo->serial_number, "0000");
 
 	sc_read_binary(card, 0, certlen, 2, 0);
 
@@ -201,7 +204,7 @@ static int sc_pkcs15emu_postecert_init(sc_pkcs15_card_t * p15card)
 	if (count < 256)
 		return SC_ERROR_INTERNAL;
 
-	certi = (unsigned char *) malloc(count);
+	certi = malloc(count);
 
 	if (!certi)
 		return SC_ERROR_OUT_OF_MEMORY;
@@ -332,7 +335,7 @@ static int sc_pkcs15emu_postecert_init(sc_pkcs15_card_t * p15card)
 	return 0;
 
 failed:
-	sc_error(card->ctx,
+	sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL,
 		 "Failed to initialize Postecert and Cnipa emulation: %s\n",
 		 sc_strerror(r));
 	return r;
