@@ -18,12 +18,14 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include "internal.h"
+#include "config.h"
+
 #include <stdlib.h>
 #include <string.h>
 
-#include <opensc/log.h>
-#include <opensc/pkcs15.h>
+#include "internal.h"
+#include "log.h"
+#include "pkcs15.h"
 
 #define MANU_ID			"SIEMENS AG"
 #define TC_CARDOS_APP_DF	"3F001002"
@@ -219,16 +221,15 @@ static int parse_EF_CardInfo(sc_pkcs15_card_t *p15card)
 	/* get the number of private keys */
 	key_num = info1[info1_len-1] | (info1[info1_len-2] << 8) |
 		  (info1[info1_len-3] << 16) | (info1[info1_len-4] << 24);
-	if (ctx->debug >= 4) {
-		sc_debug(ctx, "found %d private keys\n", (int)key_num);
-	}
+	sc_debug(ctx, SC_LOG_DEBUG_NORMAL,
+		"found %d private keys\n", (int)key_num);
 	/* set p1 to the address of the first key descriptor */
 	p1 = info1 + (info1_len - 4 - key_num * 2);
 	p2 = info2;
 	for (i=0; i<key_num; i++) {
 		u8   pinId, keyId, cert_count;
 		int  ch_cert, ca_cert, r1_cert, r2_cert = 0;
-		int  key_descr, pinType;
+		int  key_descr;
 		/* evaluate CertInfo2 */
 		cert_count = *p2++;
 		p2 += 2; /* ignore cert DF (it's always 1002) */
@@ -249,7 +250,6 @@ static int parse_EF_CardInfo(sc_pkcs15_card_t *p15card)
 		/* evaluate CertInfo1 */
 		key_descr = (p1[0] << 8) | p1[1];
 		p1 += 2;
-		pinType   = key_descr & TC_CARDOS_PIN_MASK;
 		/* create and add certificates */
 		if (ch_cert) {
 			r = create_cert_obj(p15card, ch_cert);
@@ -305,27 +305,25 @@ static int sc_pkcs15_tccardos_init_func(sc_pkcs15_card_t *p15card)
 	if (r != SC_SUCCESS)
 		return r;
 	/* set card label */
-	if (p15card->label != NULL)
-		free(p15card->label);
-	p15card->label = strdup(TC_CARDOS_LABEL);
-	if (p15card->label == NULL)
+	if (p15card->tokeninfo->label != NULL)
+		free(p15card->tokeninfo->label);
+	p15card->tokeninfo->label = strdup(TC_CARDOS_LABEL);
+	if (p15card->tokeninfo->label == NULL)
 		return SC_ERROR_OUT_OF_MEMORY;
 	/* set the manufacturer ID */
-	if (p15card->manufacturer_id != NULL)
-		free(p15card->manufacturer_id);
-	p15card->manufacturer_id = strdup(MANU_ID);
-	if (p15card->manufacturer_id == NULL)
+	if (p15card->tokeninfo->manufacturer_id != NULL)
+		free(p15card->tokeninfo->manufacturer_id);
+	p15card->tokeninfo->manufacturer_id = strdup(MANU_ID);
+	if (p15card->tokeninfo->manufacturer_id == NULL)
 		return SC_ERROR_OUT_OF_MEMORY;
 	/* set the serial number */
 	r = read_file(p15card->card, "3F002F02", gdo, &gdo_len);
 	if (r != SC_SUCCESS)
 		return SC_ERROR_INTERNAL;
 	sc_bin_to_hex(gdo + 7, 8, hex_buf, sizeof(hex_buf), 0);
-	p15card->serial_number = strdup(hex_buf);
-	if (p15card->serial_number == NULL)
+	p15card->tokeninfo->serial_number = strdup(hex_buf);
+	if (p15card->tokeninfo->serial_number == NULL)
 		return SC_ERROR_OUT_OF_MEMORY;
-	/* the TokenInfo version number */
-	p15card->version = 0;
 	/* select the application DF */
 	sc_format_path(TC_CARDOS_APP_DF, &path);
 	r = sc_select_file(card, &path, &file);
