@@ -18,8 +18,8 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include "internal.h"
-#include "pkcs15.h"
+#include "config.h"
+
 #ifdef ENABLE_OPENSSL
 #include <openssl/evp.h>
 #include <openssl/rand.h>
@@ -36,7 +36,9 @@
  * Everything seems to work fine however if the openssl one is included
  * first.
  */
+#include "internal.h"
 #include "asn1.h"
+#include "pkcs15.h"
 
 
 #ifndef ENABLE_OPENSSL
@@ -78,7 +80,7 @@ sc_pkcs15_derive_key(sc_context_t *ctx,
 
 	/* XXX: We might also encounter PBES2 here */
 	if (der_alg->algorithm != SC_ALGORITHM_PBKDF2) {
-		sc_error(ctx, "Unsupported key derivation algorithm.\n");
+		sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "Unsupported key derivation algorithm.\n");
 		return SC_ERROR_NOT_SUPPORTED;
 	}
 
@@ -92,27 +94,27 @@ sc_pkcs15_derive_key(sc_context_t *ctx,
 		iv = (u8 *) enc_alg->params;
 		break;
 	default:
-		sc_error(ctx, "Unsupported key encryption algorithm.\n");
+		sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "Unsupported key encryption algorithm.\n");
 		return SC_ERROR_NOT_SUPPORTED;
 	}
 
 	if (!iv) {
-		sc_error(ctx, "Unsupported key encryption parameters.\n");
+		sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "Unsupported key encryption parameters.\n");
 		return SC_ERROR_NOT_SUPPORTED;
 	}
 	key_len = EVP_CIPHER_key_length(cipher);
 
 	info = (struct sc_pbkdf2_params *) der_alg->params;
 	if (!info) {
-		sc_error(ctx, "Key parameters missing.\n");
+		sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "Key parameters missing.\n");
 		return SC_ERROR_INVALID_ARGUMENTS;
 	}
 	if (info->key_length && info->key_length != key_len) {
-		sc_error(ctx, "Incompatible key length.\n");
+		sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "Incompatible key length.\n");
 		return SC_ERROR_INVALID_ARGUMENTS;
 	}
 	if (key_len > sizeof(key)) {
-		sc_error(ctx, "Huge key length (%u).\n", key_len);
+		sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "Huge key length (%u).\n", key_len);
 		return SC_ERROR_INVALID_ARGUMENTS;
 	}
 
@@ -120,7 +122,7 @@ sc_pkcs15_derive_key(sc_context_t *ctx,
 			info->salt, info->salt_len,
 			info->iterations, key_len, key);
 	if (r == 0) {
-		sc_error(ctx, "Key derivation failed.\n");
+		sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "Key derivation failed.\n");
 		return SC_ERROR_INTERNAL; /* for lack of something better */
 	}
 
@@ -136,9 +138,10 @@ do_cipher(EVP_CIPHER_CTX *cipher_ctx, const u8 *in, size_t in_len,
 {
 	const u8 *end;
 	u8	*p;
-	size_t	bl, done, left, total;
+	size_t	bl, left, total;
+	int done;
 
-	*out = p = (u8 *) malloc(in_len + EVP_CIPHER_CTX_key_length(cipher_ctx));
+	*out = p = malloc(in_len + EVP_CIPHER_CTX_key_length(cipher_ctx));
 	*out_len = total = 0;
 
 	bl = EVP_CIPHER_CTX_block_size(cipher_ctx);
@@ -147,14 +150,14 @@ do_cipher(EVP_CIPHER_CTX *cipher_ctx, const u8 *in, size_t in_len,
 		if ((left = end - in) > bl)
 			left = bl;
 		if (!EVP_CipherUpdate(cipher_ctx,
-					p + total, (int *) &done,
+					p + total, &done,
 					(u8 *) in, (int)left))
 			goto fail;
 		total += done;
 		in += left;
 	}
 	if (1 || total < in_len) {
-		if (!EVP_CipherFinal(cipher_ctx, p + total, (int *) &done))
+		if (!EVP_CipherFinal(cipher_ctx, p + total, &done))
 			goto fail;
 		total += done;
 	}
@@ -223,7 +226,7 @@ sc_pkcs15_unwrap_data(sc_context_t *ctx,
 	memset(&envdata, 0, sizeof(envdata));
 	r = sc_pkcs15_decode_enveloped_data(ctx, &envdata, in, in_len);
 	if (r < 0) {
-		sc_error(ctx, "Failed to decode EnvelopedData.\n");
+		sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "Failed to decode EnvelopedData.\n");
 		return r;
 	}
 
