@@ -18,20 +18,21 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "config.h"
 
-#ifdef ENABLE_OPENSSL
-#include <openssl/crypto.h>     /* for OPENSSL_cleanse */
-#endif
-
-#include "internal.h"
 #include <stdio.h>
 #include <ctype.h>
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
+#ifdef HAVE_SYS_MMAN_H
+#include <sys/mman.h>
+#endif
+#ifdef ENABLE_OPENSSL
+#include <openssl/crypto.h>     /* for OPENSSL_cleanse */
+#endif
+
+#include "internal.h"
 
 #ifdef PACKAGE_VERSION
 static const char *sc_version = PACKAGE_VERSION;
@@ -55,7 +56,7 @@ int sc_hex_to_bin(const char *in, u8 *out, size_t *outlen)
 	while (*in != '\0') {
 		int byte = 0, nybbles = 2;
 
-		while (nybbles-- && *in && *in != ':') {
+		while (nybbles-- && *in && *in != ':' && *in != ' ') {
 			char c;
 			byte <<= 4;
 			c = *in++;
@@ -73,7 +74,7 @@ int sc_hex_to_bin(const char *in, u8 *out, size_t *outlen)
 			}
 			byte |= c;
 		}
-		if (*in == ':')
+		if (*in == ':' || *in == ' ')
 			in++;
 		if (left <= 0) {
                         err = SC_ERROR_BUFFER_TOO_SMALL;
@@ -178,58 +179,15 @@ int sc_compare_oid(const struct sc_object_id *oid1, const struct sc_object_id *o
 	return 1;
 }
 
-sc_slot_info_t * _sc_get_slot_info(sc_reader_t *reader, int slot_id)
-{
-	assert(reader != NULL);
-	if (slot_id < 0 || slot_id > reader->slot_count)
-		return NULL;
-	return &reader->slot[slot_id];
-}
-
-int sc_detect_card_presence(sc_reader_t *reader, int slot_id)
+int sc_detect_card_presence(sc_reader_t *reader)
 {
 	int r;
-	sc_slot_info_t *slot = _sc_get_slot_info(reader, slot_id);
-
-	if (slot == NULL)
-		SC_FUNC_RETURN(reader->ctx, 0, SC_ERROR_SLOT_NOT_FOUND);
-	SC_FUNC_CALLED(reader->ctx, 1);
+	SC_FUNC_CALLED(reader->ctx, SC_LOG_DEBUG_VERBOSE);
 	if (reader->ops->detect_card_presence == NULL)
-		SC_FUNC_RETURN(reader->ctx, 0, SC_ERROR_NOT_SUPPORTED);
+		SC_FUNC_RETURN(reader->ctx, SC_LOG_DEBUG_NORMAL, SC_ERROR_NOT_SUPPORTED);
 
-	r = reader->ops->detect_card_presence(reader, slot);
-	SC_FUNC_RETURN(reader->ctx, 1, r);
-}
-
-int sc_wait_for_event(sc_reader_t *readers[], int slot_id[], size_t nslots,
-                      unsigned int event_mask,
-                      int *reader, unsigned int *event, int timeout)
-{
-	sc_slot_info_t *slotp[SC_MAX_SLOTS * SC_MAX_READERS];
-	sc_context_t *ctx;
-	unsigned int j;
-	int r;
-
-	if (nslots == 0 || nslots > SC_MAX_SLOTS * SC_MAX_READERS)
-	       return SC_ERROR_INVALID_ARGUMENTS;
-	ctx = readers[0]->ctx;
-
-	SC_FUNC_CALLED(ctx, 1);
-	for (j = 0; j < nslots; j++) {
-		slotp[j] = _sc_get_slot_info(readers[j], slot_id[j]);
-
-		if (slotp[j] == NULL)
-			SC_FUNC_RETURN(ctx, 0, SC_ERROR_SLOT_NOT_FOUND);
-		/* XXX check to make sure all readers share the same operations
-		 * struct */
-	}
-
-	if (readers[0]->ops->wait_for_event == NULL)
-	       SC_FUNC_RETURN(ctx, 0, SC_ERROR_NOT_SUPPORTED);
-
-	r = readers[0]->ops->wait_for_event(readers, slotp, nslots,
-				       event_mask, reader, event, timeout);
-	SC_FUNC_RETURN(ctx, 1, r);
+	r = reader->ops->detect_card_presence(reader);
+	SC_FUNC_RETURN(reader->ctx, SC_LOG_DEBUG_NORMAL, r);
 }
 
 int sc_path_set(sc_path_t *path, int type, const u8 *id, size_t id_len, 
@@ -237,6 +195,8 @@ int sc_path_set(sc_path_t *path, int type, const u8 *id, size_t id_len,
 {
 	if (path == NULL || id == NULL || id_len == 0 || id_len > SC_MAX_PATH_SIZE)
 		return SC_ERROR_INVALID_ARGUMENTS;
+
+	memset(path, 0, sizeof(*path));
 	memcpy(path->value, id, id_len);
 	path->len   = id_len;
 	path->type  = type;
@@ -316,7 +276,7 @@ int sc_concatenate_path(sc_path_t *d, const sc_path_t *p1, const sc_path_t *p2)
 
 const char *sc_print_path(const sc_path_t *path)
 {
-	static char	buffer[SC_MAX_PATH_STRING_SIZE];
+	static char buffer[SC_MAX_PATH_STRING_SIZE + SC_MAX_AID_STRING_SIZE];
 
 	if (sc_path_print(buffer, sizeof(buffer), path) != SC_SUCCESS)
 		buffer[0] = '\0';
@@ -331,12 +291,20 @@ int sc_path_print(char *buf, size_t buflen, const sc_path_t *path)
 	if (buf == NULL || path == NULL)
 		return SC_ERROR_INVALID_ARGUMENTS;
 
-	if (buflen < path->len * 2 + 1)
+	if (buflen < path->len * 2 + path->aid.len * 2 + 1)
 		return SC_ERROR_BUFFER_TOO_SMALL;
 
 	buf[0] = '\0';
+	if (path->aid.len)   {
+		for (i = 0; i < path->aid.len; i++)
+			snprintf(buf + strlen(buf), buflen - strlen(buf), "%02x", path->aid.value[i]);
+		snprintf(buf + strlen(buf), buflen - strlen(buf), "::");
+	}
+
 	for (i = 0; i < path->len; i++)
-		snprintf(buf + 2 * i, buflen - 2 * i, "%02x", path->value[i]);
+		snprintf(buf + strlen(buf), buflen - strlen(buf), "%02x", path->value[i]);
+	if (!path->aid.len && path->type == SC_PATH_TYPE_DF_NAME)
+		snprintf(buf + strlen(buf), buflen - strlen(buf), "::");
 
 	return SC_SUCCESS;
 }
@@ -362,8 +330,13 @@ int sc_compare_path_prefix(const sc_path_t *prefix, const sc_path_t *path)
 
 const sc_path_t *sc_get_mf_path(void)
 {
-	static const sc_path_t mf_path = { {0x3f, 0x00, 0, 0, 0, 0, 0, 0,
-		0, 0, 0, 0, 0, 0, 0, 0}, 2, 0, 0, SC_PATH_TYPE_PATH};
+	static const sc_path_t mf_path = { 
+		{0x3f, 0x00, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, 2, 
+		0, 
+		0, 
+		SC_PATH_TYPE_PATH, 
+		{{0},0}
+	};
 	return &mf_path;
 }
 
@@ -406,7 +379,7 @@ int sc_file_add_acl_entry(sc_file_t *file, unsigned int operation,
 			return 0;
 	}
 
-	_new = (sc_acl_entry_t *) malloc(sizeof(sc_acl_entry_t));
+	_new = malloc(sizeof(sc_acl_entry_t));
 	if (_new == NULL)
 		return SC_ERROR_OUT_OF_MEMORY;
 	_new->method = method;
@@ -430,13 +403,13 @@ const sc_acl_entry_t * sc_file_get_acl_entry(const sc_file_t *file,
 {
 	sc_acl_entry_t *p;
 	static const sc_acl_entry_t e_never = {
-		SC_AC_NEVER, SC_AC_KEY_REF_NONE, NULL
+		SC_AC_NEVER, SC_AC_KEY_REF_NONE, {{0, 0, 0, {0}}}, NULL
 	};
 	static const sc_acl_entry_t e_none = {
-		SC_AC_NONE, SC_AC_KEY_REF_NONE, NULL
+		SC_AC_NONE, SC_AC_KEY_REF_NONE, {{0, 0, 0, {0}}}, NULL
 	};
 	static const sc_acl_entry_t e_unknown = {
-		SC_AC_UNKNOWN, SC_AC_KEY_REF_NONE, NULL
+		SC_AC_UNKNOWN, SC_AC_KEY_REF_NONE, {{0, 0, 0, {0}}}, NULL
 	};
 
 	assert(file != NULL);
@@ -639,12 +612,12 @@ int sc_file_valid(const sc_file_t *file) {
 	return file->magic == SC_FILE_MAGIC;
 }
 
-int _sc_parse_atr(sc_context_t *ctx, sc_slot_info_t *slot)
+int _sc_parse_atr(sc_reader_t *reader)
 {
-	u8 *p = slot->atr;
-	int atr_len = (int) slot->atr_len;
+	u8 *p = reader->atr.value;
+	int atr_len = (int) reader->atr.len;
 	int n_hist, x;
-	int tx[4];
+	int tx[4] = {-1, -1, -1, -1};
 	int i, FI, DI;
 	const int Fi_table[] = {
 		372, 372, 558, 744, 1116, 1488, 1860, -1,
@@ -656,16 +629,16 @@ int _sc_parse_atr(sc_context_t *ctx, sc_slot_info_t *slot)
 		-1, 1, 2, 4, 8, 16, 32, -1,
 		12, 20, -1, -1, -1, -1, -1, -1 };
 
-	slot->atr_info.hist_bytes_len = 0;
-	slot->atr_info.hist_bytes = NULL;
+	reader->atr_info.hist_bytes_len = 0;
+	reader->atr_info.hist_bytes = NULL;
 
 	if (atr_len == 0) {
-		sc_error(ctx, "empty ATR - card not present?\n");
+		sc_debug(reader->ctx, SC_LOG_DEBUG_NORMAL, "empty ATR - card not present?\n");
 		return SC_ERROR_INTERNAL;
 	}
 
 	if (p[0] != 0x3B && p[0] != 0x3F) {
-		sc_error(ctx, "invalid sync byte in ATR: 0x%02X\n", p[0]);
+		sc_debug(reader->ctx, SC_LOG_DEBUG_NORMAL, "invalid sync byte in ATR: 0x%02X\n", p[0]);
 		return SC_ERROR_INTERNAL;
 	}
 	n_hist = p[1] & 0x0F;
@@ -681,20 +654,20 @@ int _sc_parse_atr(sc_context_t *ctx, sc_slot_info_t *slot)
                         tx[i] = -1;
         }
 	if (tx[0] >= 0) {
-		slot->atr_info.FI = FI = tx[0] >> 4;
-		slot->atr_info.DI = DI = tx[0] & 0x0F;
-		slot->atr_info.Fi = Fi_table[FI];
-		slot->atr_info.f = f_table[FI];
-		slot->atr_info.Di = Di_table[DI];
+		reader->atr_info.FI = FI = tx[0] >> 4;
+		reader->atr_info.DI = DI = tx[0] & 0x0F;
+		reader->atr_info.Fi = Fi_table[FI];
+		reader->atr_info.f = f_table[FI];
+		reader->atr_info.Di = Di_table[DI];
 	} else {
-		slot->atr_info.Fi = -1;
-		slot->atr_info.f = -1;
-		slot->atr_info.Di = -1;
+		reader->atr_info.Fi = -1;
+		reader->atr_info.f = -1;
+		reader->atr_info.Di = -1;
 	}
 	if (tx[2] >= 0)
-		slot->atr_info.N = tx[3];
+		reader->atr_info.N = tx[3];
 	else
-		slot->atr_info.N = -1;
+		reader->atr_info.N = -1;
 	while (tx[3] > 0 && tx[3] & 0xF0 && atr_len > 0) {
 		x = tx[3] >> 4;
 		for (i = 0; i < 4 && atr_len > 0; i++) {
@@ -710,11 +683,29 @@ int _sc_parse_atr(sc_context_t *ctx, sc_slot_info_t *slot)
 		return 0;
 	if (n_hist > atr_len)
 		n_hist = atr_len;
-	slot->atr_info.hist_bytes_len = n_hist;
-	slot->atr_info.hist_bytes = p;
+	reader->atr_info.hist_bytes_len = n_hist;
+	reader->atr_info.hist_bytes = p;
 	return 0;
 }
 
+void *sc_mem_alloc_secure(size_t len)
+{
+    void *pointer;
+    
+    pointer = calloc(len, sizeof(unsigned char));
+    if (!pointer)
+        return NULL;
+#ifdef HAVE_SYS_MMAN_H
+    /* TODO Windows support and mprotect too */
+    /* Do not swap the memory */
+    if (mlock(pointer, len) == -1) {
+        free(pointer);
+        return NULL;
+    }
+#endif
+    return pointer;
+}
+
 void sc_mem_clear(void *ptr, size_t len)
 {
 #ifdef ENABLE_OPENSSL
@@ -726,6 +717,68 @@ void sc_mem_clear(void *ptr, size_t len)
 #endif
 }
 
+static int 
+sc_remote_apdu_allocate(struct sc_remote_data *rdata, 
+		struct sc_remote_apdu **new_rapdu)
+{
+	struct sc_remote_apdu *rapdu = NULL, *rr;
+	int counter;
+
+	if (!rdata)
+		return SC_ERROR_INVALID_ARGUMENTS;
+
+	rapdu = calloc(1, sizeof(struct sc_remote_apdu));
+	if (rapdu == NULL)
+		return SC_ERROR_OUT_OF_MEMORY;
+
+	rapdu->apdu.data = &rapdu->sbuf[0];
+	rapdu->apdu.resp = &rapdu->rbuf[0];
+	rapdu->apdu.resplen = sizeof(rapdu->rbuf);
+
+	if (new_rapdu)
+		*new_rapdu = rapdu;
+
+	if (rdata->data == NULL)   {
+		rdata->data = rapdu;
+		rdata->length = 1;
+		return SC_SUCCESS;
+	}
+
+	for (rr = rdata->data; rr->next; rr = rr->next)
+		;
+	rr->next = rapdu;
+	rdata->length++;
+
+	return SC_SUCCESS;
+}
+
+static void 
+sc_remote_apdu_free (struct sc_remote_data *rdata)
+{
+	struct sc_remote_apdu *rapdu = NULL;
+
+	if (!rdata)
+		return;
+
+	rapdu = rdata->data;
+	while(rapdu)   {
+		struct sc_remote_apdu *rr = rapdu->next;
+
+		free(rapdu);
+		rapdu = rr;
+	}
+}
+
+void sc_remote_data_init(struct sc_remote_data *rdata)
+{
+	if (!rdata)
+		return;
+	memset(rdata, 0, sizeof(struct sc_remote_data));
+
+	rdata->alloc = sc_remote_apdu_allocate;
+	rdata->free = sc_remote_apdu_free;
+}
+
 /**************************** mutex functions ************************/
 
 int sc_mutex_create(const sc_context_t *ctx, void **mutex)
