@@ -18,28 +18,45 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include "config.h"
+
 #include <stdlib.h>
 #include <string.h>
+
 #include "sc-pkcs11.h"
 
 #define DUMP_TEMPLATE_MAX	32
 
-void strcpy_bp(u8 *dst, const char *src, size_t dstsize)
+struct sc_to_cryptoki_error_conversion  {
+	const char *context;
+	int sc_error;
+	CK_RV ck_error;
+};
+
+static struct sc_to_cryptoki_error_conversion sc_to_cryptoki_error_map[]  = {
+	{ "C_GenerateKeyPair", SC_ERROR_INVALID_PIN_LENGTH, CKR_GENERAL_ERROR },
+	{NULL, 0, 0}
+};
+
+
+void strcpy_bp(u8 * dst, const char *src, size_t dstsize)
 {
 	size_t c;
 
 	if (!dst || !src || !dstsize)
 		return;
 
-	memset((char *) dst, ' ', dstsize);
-	
+	memset((char *)dst, ' ', dstsize);
+
 	c = strlen(src) > dstsize ? dstsize : strlen(src);
-	
-	memcpy((char *) dst, src, c);
+
+	memcpy((char *)dst, src, c);
 }
 
-CK_RV sc_to_cryptoki_error(int rc, int reader)
+
+static CK_RV sc_to_cryptoki_error_common(int rc)
 {
+	sc_debug(context, SC_LOG_DEBUG_NORMAL, "libopensc return value: %d (%s)\n", rc, sc_strerror(rc));
 	switch (rc) {
 	case SC_SUCCESS:
 		return CKR_OK;
@@ -54,7 +71,6 @@ CK_RV sc_to_cryptoki_error(int rc, int reader)
 	case SC_ERROR_BUFFER_TOO_SMALL:
 		return CKR_BUFFER_TOO_SMALL;
 	case SC_ERROR_CARD_NOT_PRESENT:
-		card_removed(reader);
 		return CKR_TOKEN_NOT_PRESENT;
 	case SC_ERROR_INVALID_CARD:
 		return CKR_TOKEN_NOT_RECOGNIZED;
@@ -76,100 +92,49 @@ CK_RV sc_to_cryptoki_error(int rc, int reader)
 	case SC_ERROR_INVALID_DATA:
 	case SC_ERROR_INCORRECT_PARAMETERS:
 		return CKR_DATA_INVALID;
+	case SC_ERROR_CARD_UNRESPONSIVE:
+	case SC_ERROR_READER_LOCKED:
+		return CKR_DEVICE_ERROR;
+	case SC_ERROR_READER_DETACHED:
+		return CKR_TOKEN_NOT_PRESENT;	/* Maybe CKR_DEVICE_REMOVED ? */
+	case SC_ERROR_NOT_ENOUGH_MEMORY:
+		return CKR_DEVICE_MEMORY;
+	case SC_ERROR_MEMORY_FAILURE:	/* EEPROM has failed */
+		return CKR_DEVICE_ERROR;
 	}
-	sc_debug(context, "opensc error: %s (%d)\n", sc_strerror(rc), rc);
 	return CKR_GENERAL_ERROR;
 }
 
-/* Pool */
-CK_RV pool_initialize(struct sc_pkcs11_pool *pool, int type)
+
+CK_RV sc_to_cryptoki_error(int rc, const char *ctx)
 {
-	pool->type = type;
-	pool->next_free_handle = 1;
-	pool->num_items = 0;
-	pool->head = pool->tail = NULL;
+	if (ctx)
+	{
+		int ii;
 
-	return CKR_OK;
-}
-
-CK_RV pool_insert(struct sc_pkcs11_pool *pool, void *item_ptr, CK_ULONG_PTR pHandle)
-{
-	struct sc_pkcs11_pool_item *item;
-	int handle = pool->next_free_handle++;
-
-	item = (struct sc_pkcs11_pool_item*) malloc(sizeof(struct sc_pkcs11_pool_item));
-
-	if (pHandle != NULL)
-		*pHandle = handle;
-
-	item->handle = handle;
-	item->item = item_ptr;
-	item->next = NULL;
-	item->prev = pool->tail;
-
-	if (pool->head != NULL && pool->tail != NULL) {
-		pool->tail->next = item;
-		pool->tail = item;
-	} else
-		pool->head = pool->tail = item;
-		
-	return CKR_OK;
-}
-
-CK_RV pool_find(struct sc_pkcs11_pool *pool, CK_ULONG handle, void **item_ptr)
-{
-	struct sc_pkcs11_pool_item *item;
-
-	if (context == NULL)
-		return CKR_CRYPTOKI_NOT_INITIALIZED;
-
-	for (item = pool->head; item != NULL; item = item->next) {
-		if (item->handle == handle) {
-			*item_ptr = item->item;
-				return CKR_OK;
+		for (ii = 0; sc_to_cryptoki_error_map[ii].context; ii++) {
+			if (sc_to_cryptoki_error_map[ii].sc_error != rc)
+				continue;
+			if (strcmp(sc_to_cryptoki_error_map[ii].context, ctx))
+				continue;
+			return sc_to_cryptoki_error_map[ii].ck_error;
 		}
 	}
-
-	return (pool->type == POOL_TYPE_OBJECT)? CKR_OBJECT_HANDLE_INVALID
-					       : CKR_SESSION_HANDLE_INVALID;
+	return sc_to_cryptoki_error_common(rc);
 }
 
-CK_RV pool_find_and_delete(struct sc_pkcs11_pool *pool, CK_ULONG handle, void **item_ptr)
-{
-	struct sc_pkcs11_pool_item *item;
-
-	if (context == NULL)
-		return CKR_CRYPTOKI_NOT_INITIALIZED;
-
-	for (item = pool->head; item != NULL; item = item->next) {
-		if (handle == 0 || item->handle == handle) {
-			if (item->prev) item->prev->next = item->next;
-			if (item->next) item->next->prev = item->prev;
-			if (pool->head == item) pool->head = item->next;
-			if (pool->tail == item) pool->tail = item->prev;
-
-			*item_ptr = item->item;
-			free(item);
-
-			return CKR_OK;
-		}
-	}
-
-	return (pool->type == POOL_TYPE_OBJECT)? CKR_OBJECT_HANDLE_INVALID
-					       : CKR_SESSION_HANDLE_INVALID;
-}
 
 /* Session manipulation */
-CK_RV session_start_operation(struct sc_pkcs11_session *session,
-			      int type,
-			      sc_pkcs11_mechanism_type_t *mech,
-			      struct sc_pkcs11_operation **operation)
+CK_RV session_start_operation(struct sc_pkcs11_session * session,
+			      int type, sc_pkcs11_mechanism_type_t * mech, struct sc_pkcs11_operation ** operation)
 {
 	sc_pkcs11_operation_t *op;
 
 	if (context == NULL)
 		return CKR_CRYPTOKI_NOT_INITIALIZED;
 
+	SC_FUNC_CALLED(context, SC_LOG_DEBUG_NORMAL);
+	sc_debug(context, SC_LOG_DEBUG_NORMAL, "Session 0x%lx, type %d", session->handle, type);
 	if (type < 0 || type >= SC_PKCS11_OPERATION_MAX)
 		return CKR_ARGUMENTS_BAD;
 
@@ -186,11 +151,12 @@ CK_RV session_start_operation(struct sc_pkcs11_session *session,
 	return CKR_OK;
 }
 
-CK_RV session_get_operation(struct sc_pkcs11_session *session, int type,
-				sc_pkcs11_operation_t **operation)
+CK_RV session_get_operation(struct sc_pkcs11_session * session, int type, sc_pkcs11_operation_t ** operation)
 {
 	sc_pkcs11_operation_t *op;
 
+	SC_FUNC_CALLED(context, SC_LOG_DEBUG_NORMAL);
+
 	if (type < 0 || type >= SC_PKCS11_OPERATION_MAX)
 		return CKR_ARGUMENTS_BAD;
 
@@ -203,7 +169,7 @@ CK_RV session_get_operation(struct sc_pkcs11_session *session, int type,
 	return CKR_OK;
 }
 
-CK_RV session_stop_operation(struct sc_pkcs11_session *session, int type)
+CK_RV session_stop_operation(struct sc_pkcs11_session * session, int type)
 {
 	if (type < 0 || type >= SC_PKCS11_OPERATION_MAX)
 		return CKR_ARGUMENTS_BAD;
@@ -215,9 +181,9 @@ CK_RV session_stop_operation(struct sc_pkcs11_session *session, int type)
 	return CKR_OK;
 }
 
-CK_RV attr_extract(CK_ATTRIBUTE_PTR pAttr, void *ptr, size_t *sizep)
+CK_RV attr_extract(CK_ATTRIBUTE_PTR pAttr, void *ptr, size_t * sizep)
 {
-	unsigned int	size;
+	unsigned int size;
 
 	if (sizep) {
 		size = *sizep;
@@ -227,17 +193,23 @@ CK_RV attr_extract(CK_ATTRIBUTE_PTR pAttr, void *ptr, size_t *sizep)
 	} else {
 		switch (pAttr->type) {
 		case CKA_CLASS:
-			size = sizeof(CK_OBJECT_CLASS); break;
+			size = sizeof(CK_OBJECT_CLASS);
+			break;
 		case CKA_KEY_TYPE:
-			size = sizeof(CK_KEY_TYPE); break;
+			size = sizeof(CK_KEY_TYPE);
+			break;
 		case CKA_PRIVATE:
-			size = sizeof(CK_BBOOL); break;
+			size = sizeof(CK_BBOOL);
+			break;
 		case CKA_CERTIFICATE_TYPE:
-			size = sizeof(CK_CERTIFICATE_TYPE); break;
+			size = sizeof(CK_CERTIFICATE_TYPE);
+			break;
 		case CKA_MODULUS_BITS:
-			size = sizeof(CK_ULONG); break;
+			size = sizeof(CK_ULONG);
+			break;
 		case CKA_OBJECT_ID:
-			size = sizeof(struct sc_object_id); break;
+			size = sizeof(struct sc_object_id);
+			break;
 		default:
 			return CKR_FUNCTION_FAILED;
 		}
@@ -248,10 +220,9 @@ CK_RV attr_extract(CK_ATTRIBUTE_PTR pAttr, void *ptr, size_t *sizep)
 	return CKR_OK;
 }
 
-CK_RV attr_find(CK_ATTRIBUTE_PTR pTemplate, CK_ULONG ulCount,
-			CK_ULONG type, void *ptr, size_t *sizep)
+CK_RV attr_find(CK_ATTRIBUTE_PTR pTemplate, CK_ULONG ulCount, CK_ULONG type, void *ptr, size_t * sizep)
 {
-	unsigned int	n;
+	unsigned int n;
 
 	for (n = 0; n < ulCount; n++, pTemplate++) {
 		if (pTemplate->type == type)
@@ -264,11 +235,10 @@ CK_RV attr_find(CK_ATTRIBUTE_PTR pTemplate, CK_ULONG ulCount,
 }
 
 CK_RV attr_find2(CK_ATTRIBUTE_PTR pTemp1, CK_ULONG ulCount1,
-		CK_ATTRIBUTE_PTR pTemp2, CK_ULONG ulCount2,
-		CK_ULONG type, void *ptr, size_t *sizep)
+		 CK_ATTRIBUTE_PTR pTemp2, CK_ULONG ulCount2, CK_ULONG type, void *ptr, size_t * sizep)
 {
 	CK_RV rv;
-	
+
 	rv = attr_find(pTemp1, ulCount1, type, ptr, sizep);
 	if (rv != CKR_OK)
 		rv = attr_find(pTemp2, ulCount2, type, ptr, sizep);
@@ -276,10 +246,9 @@ CK_RV attr_find2(CK_ATTRIBUTE_PTR pTemp1, CK_ULONG ulCount1,
 	return rv;
 }
 
-CK_RV attr_find_ptr(CK_ATTRIBUTE_PTR pTemplate, CK_ULONG ulCount,
-		CK_ULONG type, void **ptr, size_t *sizep)
+CK_RV attr_find_ptr(CK_ATTRIBUTE_PTR pTemplate, CK_ULONG ulCount, CK_ULONG type, void **ptr, size_t * sizep)
 {
-	unsigned int	n;
+	unsigned int n;
 
 	for (n = 0; n < ulCount; n++, pTemplate++) {
 		if (pTemplate->type == type)
@@ -295,10 +264,9 @@ CK_RV attr_find_ptr(CK_ATTRIBUTE_PTR pTemplate, CK_ULONG ulCount,
 	return CKR_OK;
 }
 
-CK_RV attr_find_var(CK_ATTRIBUTE_PTR pTemplate, CK_ULONG ulCount,
-		CK_ULONG type, void *ptr, size_t *sizep)
+CK_RV attr_find_var(CK_ATTRIBUTE_PTR pTemplate, CK_ULONG ulCount, CK_ULONG type, void *ptr, size_t * sizep)
 {
-	unsigned int	n;
+	unsigned int n;
 
 	for (n = 0; n < ulCount; n++, pTemplate++) {
 		if (pTemplate->type == type)
@@ -311,19 +279,20 @@ CK_RV attr_find_var(CK_ATTRIBUTE_PTR pTemplate, CK_ULONG ulCount,
 	return attr_extract(pTemplate, ptr, sizep);
 }
 
-void load_pkcs11_parameters(struct sc_pkcs11_config *conf, sc_context_t *ctx)
+void load_pkcs11_parameters(struct sc_pkcs11_config *conf, sc_context_t * ctx)
 {
 	scconf_block *conf_block = NULL;
+	char *unblock_style = NULL;
 
 	/* Set defaults */
 	conf->plug_and_play = 1;
 	conf->max_virtual_slots = 16;
 	conf->slots_per_card = 4;
 	conf->hide_empty_tokens = 1;
-	conf->lock_login = 1;
-	conf->cache_pins = 1;
-	conf->soft_keygen_allowed = 0;
-
+	conf->lock_login = 0;
+	conf->pin_unblock_style = SC_PKCS11_PIN_UNBLOCK_NOT_ALLOWED;
+	conf->create_puk_slot = 0;
+	conf->zero_ckaid_for_ca_certs = 0;
 
 	conf_block = sc_get_conf_block(ctx, "pkcs11", NULL, 1);
 	if (!conf_block)
@@ -332,11 +301,24 @@ void load_pkcs11_parameters(struct sc_pkcs11_config *conf, sc_context_t *ctx)
 	/* contains the defaults, if there is a "pkcs11" config block */
 	conf->plug_and_play = scconf_get_bool(conf_block, "plug_and_play", conf->plug_and_play);
 	conf->max_virtual_slots = scconf_get_int(conf_block, "max_virtual_slots", conf->max_virtual_slots);
-	/*XXX: rename the option in 0.12+ */
-	conf->slots_per_card = scconf_get_int(conf_block, "num_slots", conf->slots_per_card);
 	conf->slots_per_card = scconf_get_int(conf_block, "slots_per_card", conf->slots_per_card);
 	conf->hide_empty_tokens = scconf_get_bool(conf_block, "hide_empty_tokens", conf->hide_empty_tokens);
 	conf->lock_login = scconf_get_bool(conf_block, "lock_login", conf->lock_login);
-	conf->cache_pins = scconf_get_bool(conf_block, "cache_pins", conf->cache_pins);
-	conf->soft_keygen_allowed = scconf_get_bool(conf_block, "soft_keygen_allowed", conf->soft_keygen_allowed);
+
+	unblock_style = (char *)scconf_get_str(conf_block, "user_pin_unblock_style", NULL);
+	if (unblock_style && !strcmp(unblock_style, "set_pin_in_unlogged_session"))
+		conf->pin_unblock_style = SC_PKCS11_PIN_UNBLOCK_UNLOGGED_SETPIN;
+	else if (unblock_style && !strcmp(unblock_style, "set_pin_in_specific_context"))
+		conf->pin_unblock_style = SC_PKCS11_PIN_UNBLOCK_SCONTEXT_SETPIN;
+	else if (unblock_style && !strcmp(unblock_style, "init_pin_in_so_session"))
+		conf->pin_unblock_style = SC_PKCS11_PIN_UNBLOCK_SO_LOGGED_INITPIN;
+	
+	conf->create_puk_slot = scconf_get_bool(conf_block, "create_puk_slot", conf->create_puk_slot);
+	conf->zero_ckaid_for_ca_certs = scconf_get_bool(conf_block, "zero_ckaid_for_ca_certs", conf->zero_ckaid_for_ca_certs);
+
+	sc_debug(ctx, SC_LOG_DEBUG_NORMAL, "PKCS#11 options: plug_and_play=%d max_virtual_slots=%d slots_per_card=%d "
+		 "hide_empty_tokens=%d lock_login=%d pin_unblock_style=%d zero_ckaid_for_ca_certs=%d",
+		 conf->plug_and_play, conf->max_virtual_slots, conf->slots_per_card,
+		 conf->hide_empty_tokens, conf->lock_login, conf->pin_unblock_style,
+		 conf->zero_ckaid_for_ca_certs);
 }
