@@ -16,36 +16,34 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "config.h"
+
 #include <sys/types.h>
 #include <stdlib.h>
 #include <string.h>
-#include <opensc/opensc.h>
-#include <opensc/cardctl.h>
-#include <opensc/log.h>
+
+#include "libopensc/opensc.h"
+#include "libopensc/cardctl.h"
+#include "libopensc/log.h"
 #include "pkcs15-init.h"
 #include "profile.h"
 
 /* delete a EF/DF if present. This function does not return an 
  * error if the requested file is not present. 
  */
-static int asepcos_cond_delete(sc_profile_t *pro, sc_card_t *card,
+static int asepcos_cond_delete(sc_profile_t *pro, sc_pkcs15_card_t *p15card,
 	const sc_path_t *path)
 {
 	int r;
 	sc_file_t *tfile = NULL;
 
-	sc_ctx_suppress_errors_on(card->ctx);
-	r = sc_select_file(card, path, &tfile);
-	sc_ctx_suppress_errors_off(card->ctx);
+	r = sc_select_file(p15card->card, path, &tfile);
 	if (r == SC_SUCCESS) {
-		r = sc_pkcs15init_authenticate(pro, card, tfile, SC_AC_OP_DELETE_SELF);
+		r = sc_pkcs15init_authenticate(pro, p15card, tfile, SC_AC_OP_DELETE_SELF);
 		sc_file_free(tfile);
 		if (r != SC_SUCCESS)
 			return r;
-		r = sc_delete_file(card, path);
+		r = sc_delete_file(p15card->card, path);
 	} else if (r == SC_ERROR_FILE_NOT_FOUND)
 		r = SC_SUCCESS;
 	return r;
@@ -58,20 +56,19 @@ static int asepcos_cond_delete(sc_profile_t *pro, sc_card_t *card,
  * @param  card     sc_card_t object to use
  * @return SC_SUCCESS on success and an error code otherwise
  */
-static int asepcos_check_verify_tpin(sc_profile_t *profile, sc_card_t *card)
+static int asepcos_check_verify_tpin(sc_profile_t *profile, sc_pkcs15_card_t *p15card)
 {
+	struct sc_context *ctx = p15card->card->ctx;
 	int r;
 	sc_path_t path;
+
 	/* check whether the file with the transport PIN exists */
 	sc_format_path("3f000001", &path);
-	sc_ctx_suppress_errors_on(card->ctx);
-	r = sc_select_file(card, &path, NULL);
-	sc_ctx_suppress_errors_off(card->ctx);
+	r = sc_select_file(p15card->card, &path, NULL);
 	if (r == SC_SUCCESS) {
 		/* try to verify the transport key */
-		u8     pbuf[64];
-		size_t psize = sizeof(pbuf);
 		sc_file_t *tfile = NULL;
+
 		sc_format_path("3f00", &path);
 		r = sc_profile_get_file_by_path(profile, sc_get_mf_path(), &tfile);
 		if (r != SC_SUCCESS)
@@ -79,25 +76,11 @@ static int asepcos_check_verify_tpin(sc_profile_t *profile, sc_card_t *card)
 		/* we need to temporarily disable the SC_CARD_CAP_USE_FCI_AC
 		 * flag to trick sc_pkcs15init_authenticate() to use access 
 		 * information form the profile file */
-		card->caps &= ~SC_CARD_CAP_USE_FCI_AC;
-		r = sc_pkcs15init_authenticate(profile, card, tfile, SC_AC_OP_CRYPTO);
-		card->caps |=  SC_CARD_CAP_USE_FCI_AC;
+		p15card->card->caps &= ~SC_CARD_CAP_USE_FCI_AC;
+		r = sc_pkcs15init_authenticate(profile, p15card, tfile, SC_AC_OP_CRYPTO);
+		p15card->card->caps |=  SC_CARD_CAP_USE_FCI_AC;
 		sc_file_free(tfile);
-		if (r != SC_SUCCESS) {
-			sc_error(card->ctx, "unable to authenticate");
-			return r;
-		}
-		/* store the transport key as a PIN */
-		r = sc_keycache_get_key(&path, SC_AC_AUT, 0, pbuf, psize);
-		if (r < 0) {
-			sc_error(card->ctx, "unable to get transport key");
-			return r;
-		}
-		r = sc_keycache_put_key(&path, SC_AC_CHV, 0, pbuf, (size_t)r);
-		if (r != SC_SUCCESS) {
-			sc_error(card->ctx, "unable to store transport key");
-			return r;
-		}
+		SC_TEST_RET(ctx, SC_LOG_DEBUG_NORMAL, r, "unable to authenticate for 'CRYPTO' operation");
 	}
 	return SC_SUCCESS;
 }
@@ -109,7 +92,7 @@ static int asepcos_check_verify_tpin(sc_profile_t *profile, sc_card_t *card)
  *                  erased.
  * @return SC_SUCCESS on success and an error code otherwise
  */
-static int asepcos_erase(struct sc_profile *profile, sc_card_t *card)
+static int asepcos_erase(struct sc_profile *profile, sc_pkcs15_card_t *p15card)
 {
 	int r;
 	sc_path_t path;
@@ -119,17 +102,19 @@ static int asepcos_erase(struct sc_profile *profile, sc_card_t *card)
 	 *         pkcs15 application.
 	 */
 	/* Check wether a transport exists and verify it if present */
-	r = asepcos_check_verify_tpin(profile, card);
+	
+	p15card->opts.use_pin_cache = 1;
+	r = asepcos_check_verify_tpin(profile, p15card);
 	if (r != SC_SUCCESS)
 		return r;
 	/* EF(DIR) */
 	sc_format_path("3f002f00", &path);
-	r = asepcos_cond_delete(profile, card, &path);
+	r = asepcos_cond_delete(profile, p15card, &path);
 	if (r != SC_SUCCESS)
 		return r;
 	/* DF(PKCS15) */
 	sc_format_path("3f005015", &path);
-	r = asepcos_cond_delete(profile, card, &path);
+	r = asepcos_cond_delete(profile, p15card, &path);
 	if (r != SC_SUCCESS)
 		return r;
 	
@@ -143,15 +128,17 @@ static int asepcos_erase(struct sc_profile *profile, sc_card_t *card)
  * @param  df       sc_file_t with the application DF to create 
  * @return SC_SUCCESS on success and an error value otherwise
  */
-static int asepcos_create_dir(sc_profile_t *profile, sc_card_t *card,
+static int asepcos_create_dir(sc_profile_t *profile, sc_pkcs15_card_t *p15card,
 	sc_file_t *df)
 {
 	int r;
 	static const u8 pa_acl[] = {0x80,0x01,0x5f,0x90,0x00};
 	sc_file_t *tfile;
+	sc_context_t *ctx = p15card->card->ctx;
 
+	SC_FUNC_CALLED(ctx, SC_LOG_DEBUG_NORMAL);
 	/* Check wether a transport exists and verify it if present */
-	r = asepcos_check_verify_tpin(profile, card);
+	r = asepcos_check_verify_tpin(profile, p15card);
 	if (r != SC_SUCCESS)
 		return r;
 	/* As we don't know whether or not a SO-PIN is used to protect the AC
@@ -170,9 +157,9 @@ static int asepcos_create_dir(sc_profile_t *profile, sc_card_t *card,
 		return r;
 	}
 	/* create application DF */
-	r = sc_pkcs15init_create_file(profile, card, tfile);
+	r = sc_pkcs15init_create_file(profile, p15card, tfile);
 	sc_file_free(tfile);
-	return r;
+	SC_FUNC_RETURN(ctx, SC_LOG_DEBUG_VERBOSE, r);
 }
 
 
@@ -180,8 +167,8 @@ static int asepcos_create_dir(sc_profile_t *profile, sc_card_t *card,
  * determined when the PIN is created. This is just helper function to
  * determine the next best file id of the PIN file.
  */
-static int asepcos_select_pin_reference(sc_profile_t *profile, sc_card_t *card,
-	sc_pkcs15_pin_info_t *pinfo)
+static int asepcos_select_pin_reference(sc_profile_t *profile, 
+		sc_pkcs15_card_t *p15card, sc_pkcs15_pin_info_t *pinfo)
 {
 	if (pinfo->flags & SC_PKCS15_PIN_FLAG_SO_PIN)
 		return SC_SUCCESS;
@@ -219,7 +206,7 @@ static int asepcos_pinid_to_akn(sc_card_t *card, int fileid, int *akn)
 	if (r != SC_SUCCESS)
 		return r;
 	if (nfile->prop_attr == NULL || nfile->prop_attr_len != 11) {
-		sc_error(card->ctx, "unable to determine AKN");
+		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "unable to determine AKN");
 		sc_file_free(nfile);
 		return SC_ERROR_INTERNAL;
 	}
@@ -244,7 +231,7 @@ static int asepcos_do_store_pin(sc_profile_t *profile, sc_card_t *card,
 	*p++ = pinid & 0xff;
 	/* pin length */
 	if (pinlen < 4 || pinlen > 16) {
-		sc_error(card->ctx, "invalid PIN length");
+		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "invalid PIN length");
 		return SC_ERROR_INVALID_ARGUMENTS;
 	}
 	*p++ = 0x00;
@@ -310,7 +297,7 @@ static int asepcos_do_store_pin(sc_profile_t *profile, sc_card_t *card,
 	r = sc_create_file(card, nfile);
 	sc_file_free(nfile);
 	if (r != SC_SUCCESS) {
-		sc_error(card->ctx, "unable to create PIN file");
+		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "unable to create PIN file");
 		return r;
 	}
 	/* get AKN of the newly created PIN  */
@@ -340,13 +327,6 @@ static int have_onepin(sc_profile_t *profile)
                 return 0;
 }
 
-static void asepcos_fix_pin_reference(sc_pkcs15_pin_info_t *pinfo)
-{
-	if (pinfo->flags & SC_PKCS15_PIN_FLAG_SO_PIN)
-		sc_keycache_set_pin_name(&pinfo->path, pinfo->reference, SC_PKCS15INIT_SO_PIN);
-	else
-		sc_keycache_set_pin_name(&pinfo->path, pinfo->reference, SC_PKCS15INIT_USER_PIN);
-}
 
 /* create PIN and, if specified, PUK files
  * @param  profile  profile information for this card
@@ -358,15 +338,18 @@ static void asepcos_fix_pin_reference(sc_pkcs15_pin_info_t *pinfo)
  * @param  puk_len  PUK length (optional)
  * @return SC_SUCCESS on success and an error code otherwise
  */
-static int asepcos_create_pin(sc_profile_t *profile, sc_card_t *card,
+static int asepcos_create_pin(sc_profile_t *profile, sc_pkcs15_card_t *p15card,
 	sc_file_t *df, sc_pkcs15_object_t *pin_obj,
 	const u8 *pin, size_t pin_len, const u8 *puk, size_t puk_len)
 {
 	sc_pkcs15_pin_info_t *pinfo = (sc_pkcs15_pin_info_t *) pin_obj->data;
+	struct sc_card *card = p15card->card;
 	int       r, pid, puk_id;
 	sc_path_t tpath = df->path;
 	sc_file_t *tfile = NULL;
+	sc_context_t *ctx = p15card->card->ctx;
 
+	SC_FUNC_CALLED(ctx, SC_LOG_DEBUG_NORMAL);
 	if (!pin || !pin_len)
 		return SC_ERROR_INVALID_ARGUMENTS;
 
@@ -375,13 +358,13 @@ static int asepcos_create_pin(sc_profile_t *profile, sc_card_t *card,
 	/* get the ACL of the application DF */
 	r = sc_select_file(card, &df->path, &tfile);
 	if (r != SC_SUCCESS)
-		return r;
+		SC_FUNC_RETURN(ctx, SC_LOG_DEBUG_VERBOSE, r);
 	/* verify the PIN protecting the CREATE acl (if necessary) */
-	r = sc_pkcs15init_authenticate(profile, card, tfile, SC_AC_OP_CREATE);
+	r = sc_pkcs15init_authenticate(profile, p15card, tfile, SC_AC_OP_CREATE);
 	sc_file_free(tfile);
 	if (r != SC_SUCCESS) {
-		sc_error(card->ctx, "unable to create PIN file, insufficent rights");
-		return r;
+		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "unable to create PIN file, insufficent rights");
+		SC_FUNC_RETURN(ctx, SC_LOG_DEBUG_VERBOSE, r);
 	}
 
 	do {
@@ -392,15 +375,13 @@ static int asepcos_create_pin(sc_profile_t *profile, sc_card_t *card,
 		 * is already used */
 		r = sc_append_file_id(&pin_path, pid & 0xff);
 		if (r != SC_SUCCESS)
-			return r;
-		sc_ctx_suppress_errors_on(card->ctx);
+			SC_FUNC_RETURN(ctx, SC_LOG_DEBUG_VERBOSE, r);
 		r = sc_select_file(card, &pin_path, NULL);
-		sc_ctx_suppress_errors_off(card->ctx);
 		if (r == SC_SUCCESS)
 			pid += 2;
 		else if (r != SC_ERROR_FILE_NOT_FOUND) {
-			sc_error(card->ctx, "error selecting PIN file");
-			return r;
+			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "error selecting PIN file");
+			SC_FUNC_RETURN(ctx, SC_LOG_DEBUG_VERBOSE, r);
 		}
 	} while (r != SC_ERROR_FILE_NOT_FOUND);
 
@@ -414,19 +395,20 @@ static int asepcos_create_pin(sc_profile_t *profile, sc_card_t *card,
 			sc_profile_get_pin_info(profile, SC_PKCS15INIT_SO_PUK, &puk_info);
 		else
 			sc_profile_get_pin_info(profile, SC_PKCS15INIT_USER_PUK, &puk_info);
+
 		/* If a PUK we use "file id of the PIN" + 1  as the file id
 		 * of the PUK.
 		 */
 		puk_id = pid + 1;
 		r = asepcos_do_store_pin(profile, card, &puk_info, puk, puk_len, 0, puk_id);
 		if (r != SC_SUCCESS) 
-			return r;
+			SC_FUNC_RETURN(ctx, SC_LOG_DEBUG_VERBOSE, r);
 	} else
 		puk_id = 0;
 
 	r = asepcos_do_store_pin(profile, card, pinfo, pin, pin_len, puk_id, pid);
 	if (r != SC_SUCCESS)
-		return r;
+		SC_FUNC_RETURN(ctx, SC_LOG_DEBUG_VERBOSE, r);
 
 #if 1
 	if (pinfo->flags & SC_PKCS15_PIN_FLAG_SO_PIN || 
@@ -436,24 +418,18 @@ static int asepcos_create_pin(sc_profile_t *profile, sc_card_t *card,
 		 * first USER PIN has been set we can tighten the ACLs of 
 		 * the application DF. 
 		 */
-		sc_debug(card->ctx, "finalizing application DF");
-
-		/* first we need to fix the reference to pin in the key
-		 * keycache as sc_pkcs15init_fixup_file() will otherwise
-		 * mess up the ACLs */
-		asepcos_fix_pin_reference(pinfo);
-
+		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "finalizing application DF");
 		r = sc_select_file(card, &df->path, NULL);
 		if (r != SC_SUCCESS)
-			return r;
+			SC_FUNC_RETURN(ctx, SC_LOG_DEBUG_VERBOSE, r);
 		/* remove symbolic references from the ACLs */
-		r = sc_pkcs15init_fixup_file(profile, df);
+		r = sc_pkcs15init_fixup_file(profile, p15card, df);
 		if (r != SC_SUCCESS)
-			return r;
+			SC_FUNC_RETURN(ctx, SC_LOG_DEBUG_VERBOSE, r);
 		r = sc_card_ctl(card, SC_CARDCTL_ASEPCOS_SET_SATTR, df);
 		if (r != SC_SUCCESS) {
-			sc_error(card->ctx, "unable to change the security attributes");
-			return r;
+			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "unable to change the security attributes");
+			SC_FUNC_RETURN(ctx, SC_LOG_DEBUG_VERBOSE, r);
 		}
 		/* finally activate the application DF (fix ACLs) */
 		/* 1. select MF */
@@ -465,8 +441,8 @@ static int asepcos_create_pin(sc_profile_t *profile, sc_card_t *card,
 		st.is_ef  = 0;
 		r = sc_card_ctl(card, SC_CARDCTL_ASEPCOS_ACTIVATE_FILE, &st);
 		if (r != SC_SUCCESS) {
-			sc_error(card->ctx, "unable to activate DF");
-			return r;
+			sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "unable to activate DF");
+			SC_FUNC_RETURN(ctx, SC_LOG_DEBUG_VERBOSE, r);
 		}
 	}
 #endif
@@ -480,7 +456,7 @@ static int asepcos_create_pin(sc_profile_t *profile, sc_card_t *card,
 		return r;
 	pinfo->path = tpath;
 #endif
-	return r;
+	SC_FUNC_RETURN(ctx, SC_LOG_DEBUG_VERBOSE, r);
 }
 
 /* internal wrapper for sc_pkcs15init_authenticate()
@@ -490,21 +466,21 @@ static int asepcos_create_pin(sc_profile_t *profile, sc_card_t *card,
  * @param  op       the required access method
  * @return SC_SUCCESS on success and an error code otherwise
  */
-static int asepcos_do_authenticate(sc_profile_t *profile, sc_card_t *card, 
+static int asepcos_do_authenticate(sc_profile_t *profile, sc_pkcs15_card_t *p15card, 
 	const sc_path_t *path, int op)
 {
 	int r;
 	sc_file_t *prkey = NULL;
 	r = sc_profile_get_file_by_path(profile, path, &prkey);
 	if (r != SC_SUCCESS) {
-		sc_error(card->ctx, "unable to find file in profile");
+		sc_debug(p15card->card->ctx, SC_LOG_DEBUG_NORMAL, "unable to find file in profile");
 		return r;
 	}
 
-	r = sc_pkcs15init_authenticate(profile, card, prkey, op);
+	r = sc_pkcs15init_authenticate(profile, p15card, prkey, op);
 	sc_file_free(prkey);
 	if (r != SC_SUCCESS) {
-		sc_error(card->ctx, "unable to authenticate");
+		sc_debug(p15card->card->ctx, SC_LOG_DEBUG_NORMAL, "unable to authenticate");
 		return r;
 	}
 	return SC_SUCCESS;	
@@ -539,7 +515,7 @@ static int asepcos_do_create_key(sc_card_t *card, size_t ksize, int fileid,
 	*p++ = 0x82;
 	p   += 2;
 	/* file id */
-	*p++ = (fileid >> 8) && 0xff;
+	*p++ = (fileid >> 8) & 0xff;
 	*p++ = fileid & 0xff;
 	/* key size */
 	*p++ = (ksize >> 8) & 0xff;
@@ -565,7 +541,7 @@ static int asepcos_do_create_key(sc_card_t *card, size_t ksize, int fileid,
 	nfile->id   = fileid & 0xffff;
 	r = sc_file_set_prop_attr(nfile, buf, p - buf);
 	if (r != SC_SUCCESS) {
-		sc_error(card->ctx, "unable to set key prop. attributes");
+		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "unable to set key prop. attributes");
 		sc_file_free(nfile);
 		return r;
 	}
@@ -573,7 +549,7 @@ static int asepcos_do_create_key(sc_card_t *card, size_t ksize, int fileid,
 	r = sc_create_file(card, nfile);
 	sc_file_free(nfile);
 	if (r != SC_SUCCESS) {
-		sc_error(card->ctx, "unable to create key file");
+		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "unable to create key file");
 		return r;
 	}
 	return r;
@@ -581,7 +557,7 @@ static int asepcos_do_create_key(sc_card_t *card, size_t ksize, int fileid,
 
 /* creates a key file 
  */
-static int asepcos_create_key(sc_profile_t *profile, sc_card_t *card,
+static int asepcos_create_key(sc_profile_t *profile, sc_pkcs15_card_t *p15card,
 	sc_pkcs15_object_t *obj)
 {
 	sc_pkcs15_prkey_info_t *kinfo = (sc_pkcs15_prkey_info_t *) obj->data;
@@ -596,10 +572,12 @@ static int asepcos_create_key(sc_profile_t *profile, sc_card_t *card,
 		/* the key is proctected by a PIN */
 		/* XXX use the pkcs15 structures for this */
 		sc_cardctl_asepcos_akn2fileid_t st;
-		st.akn = sc_keycache_find_named_pin(NULL, SC_PKCS15INIT_USER_PIN);
-		r = sc_card_ctl(card, SC_CARDCTL_ASEPCOS_AKN2FILEID, &st);
+        
+		st.akn = sc_pkcs15init_get_pin_reference(p15card, profile,
+			                        SC_AC_SYMBOLIC, SC_PKCS15INIT_USER_PIN);
+		r = sc_card_ctl(p15card->card, SC_CARDCTL_ASEPCOS_AKN2FILEID, &st);
 		if (r != SC_SUCCESS) {
-			sc_error(card->ctx, "unable to determine file id of the PIN");
+			sc_debug(p15card->card->ctx, SC_LOG_DEBUG_NORMAL, "unable to determine file id of the PIN");
 			return r;
 		}
 		afileid = st.fileid;
@@ -612,7 +590,7 @@ static int asepcos_create_key(sc_profile_t *profile, sc_card_t *card,
 #endif
 
 	/* authenticate if necessary */
-	r = asepcos_do_authenticate(profile, card, &profile->df_info->file->path, SC_AC_OP_CREATE);
+	r = asepcos_do_authenticate(profile, p15card, &profile->df_info->file->path, SC_AC_OP_CREATE);
 	if (r != SC_SUCCESS) 
 		return r;
 
@@ -662,17 +640,19 @@ static int asepcos_create_key(sc_profile_t *profile, sc_card_t *card,
 		*p++ = 0x00;
 	}
 
-	r = asepcos_do_create_key(card, kinfo->modulus_length, fileid, buf, p - buf);
+	r = asepcos_do_create_key(p15card->card, kinfo->modulus_length, fileid, buf, p - buf);
 	if (r != SC_SUCCESS) {
-		sc_error(card->ctx, "unable to create private key file");
+		sc_debug(p15card->card->ctx, SC_LOG_DEBUG_NORMAL, "unable to create private key file");
 		return r;
 	}
+	
+	kinfo->key_reference = fileid & 0xFF;
 	return r;
 }
 
 /* stores a rsa private key in a internal EF
  */
-static int asepcos_do_store_rsa_key(sc_card_t *card, sc_profile_t *profile,
+static int asepcos_do_store_rsa_key(sc_pkcs15_card_t *p15card, sc_profile_t *profile,
 	sc_pkcs15_object_t *obj, sc_pkcs15_prkey_info_t *kinfo,
 	struct sc_pkcs15_prkey_rsa *key)
 {
@@ -683,19 +663,20 @@ static int asepcos_do_store_rsa_key(sc_card_t *card, sc_profile_t *profile,
 
 	/* authenticate if necessary */
 	if (obj->auth_id.len != 0) {
-		r = asepcos_do_authenticate(profile, card, &kinfo->path, SC_AC_OP_UPDATE);
+		r = asepcos_do_authenticate(profile, p15card, &kinfo->path, SC_AC_OP_UPDATE);
 		if (r != SC_SUCCESS) 
 			return r;
 	}
 
 	/* select the rsa private key */
+	memset(&tpath, 0, sizeof(sc_path_t));
 	tpath.type = SC_PATH_TYPE_FILE_ID;
 	tpath.len  = 2;
 	tpath.value[0] = kinfo->path.value[kinfo->path.len-2];
 	tpath.value[1] = kinfo->path.value[kinfo->path.len-1];
-	r = sc_select_file(card, &tpath, NULL);
+	r = sc_select_file(p15card->card, &tpath, NULL);
 	if (r != SC_SUCCESS) {
-		sc_error(card->ctx, "unable to select rsa key file");
+		sc_debug(p15card->card->ctx, SC_LOG_DEBUG_NORMAL, "unable to select rsa key file");
 		return r;
 	}
 
@@ -732,9 +713,9 @@ static int asepcos_do_store_rsa_key(sc_card_t *card, sc_profile_t *profile,
 	ckdata.data    = buf;
 	ckdata.datalen = p - buf;
 
-	r = sc_card_ctl(card, SC_CARDCTL_ASEPCOS_CHANGE_KEY, &ckdata);
+	r = sc_card_ctl(p15card->card, SC_CARDCTL_ASEPCOS_CHANGE_KEY, &ckdata);
 	if (r != SC_SUCCESS) {
-		sc_error(card->ctx, "unable to change key data");
+		sc_debug(p15card->card->ctx, SC_LOG_DEBUG_NORMAL, "unable to change key data");
 		return r;
 	}
 	
@@ -748,17 +729,17 @@ static int asepcos_do_store_rsa_key(sc_card_t *card, sc_profile_t *profile,
  * @param  key      the private key
  * @return SC_SUCCESS on success and an error code otherwise
  */
-static int asepcos_store_key(sc_profile_t *profile, sc_card_t *card,
+static int asepcos_store_key(sc_profile_t *profile, sc_pkcs15_card_t *p15card,
 	sc_pkcs15_object_t *obj, sc_pkcs15_prkey_t *key)
 {
 	sc_pkcs15_prkey_info_t *kinfo = (sc_pkcs15_prkey_info_t *) obj->data;
 
 	if (obj->type != SC_PKCS15_TYPE_PRKEY_RSA) {
-		sc_error(card->ctx, "only RSA is currently supported");
+		sc_debug(p15card->card->ctx, SC_LOG_DEBUG_NORMAL, "only RSA is currently supported");
 		return SC_ERROR_NOT_SUPPORTED;
 	}
 
-	return asepcos_do_store_rsa_key(card, profile, obj, kinfo, &key->u.rsa);
+	return asepcos_do_store_rsa_key(p15card, profile, obj, kinfo, &key->u.rsa);
 }
 
 /* Generates a new (RSA) key pair using an existing key file.
@@ -768,29 +749,31 @@ static int asepcos_store_key(sc_profile_t *profile, sc_card_t *card,
  * @param  pukkey   OUT the newly created public key
  * @return SC_SUCCESS on success and an error code otherwise
  */
-static int asepcos_generate_key(sc_profile_t *profile, sc_card_t *card,
+static int asepcos_generate_key(sc_profile_t *profile, sc_pkcs15_card_t *p15card,
 	sc_pkcs15_object_t *obj, sc_pkcs15_pubkey_t *pubkey)
 {
 	int r;
 	sc_pkcs15_prkey_info_t *kinfo = (sc_pkcs15_prkey_info_t *) obj->data;
+	sc_card_t *card = p15card->card;
 	sc_apdu_t apdu;
 	sc_path_t tpath;
 	u8  rbuf[SC_MAX_APDU_BUFFER_SIZE],
 	    sbuf[SC_MAX_APDU_BUFFER_SIZE];
 
 	/* authenticate if necessary */
-	r = asepcos_do_authenticate(profile, card, &kinfo->path, SC_AC_OP_UPDATE);
+	r = asepcos_do_authenticate(profile, p15card, &kinfo->path, SC_AC_OP_UPDATE);
 	if (r != SC_SUCCESS) 
 		return r;
 
 	/* select the rsa private key */
+	memset(&tpath, 0, sizeof(sc_path_t));
 	tpath.type = SC_PATH_TYPE_FILE_ID;
 	tpath.len  = 2;
 	tpath.value[0] = kinfo->path.value[kinfo->path.len-2];
 	tpath.value[1] = kinfo->path.value[kinfo->path.len-1];
 	r = sc_select_file(card, &tpath, NULL);
 	if (r != SC_SUCCESS) {
-		sc_error(card->ctx, "unable to select rsa key file");
+		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "unable to select rsa key file");
 		return r;
 	}
 
@@ -807,9 +790,9 @@ static int asepcos_generate_key(sc_profile_t *profile, sc_card_t *card,
 	apdu.resp    = rbuf;
 
 	r = sc_transmit_apdu(card, &apdu);
-	SC_TEST_RET(card->ctx, r, "APDU transmit failed");
+	SC_TEST_RET(card->ctx, SC_LOG_DEBUG_NORMAL, r, "APDU transmit failed");
 	if (apdu.sw1 != 0x90 || apdu.sw2 != 0x00) {
-		sc_error(card->ctx, "error creating key");
+		sc_debug(card->ctx, SC_LOG_DEBUG_NORMAL, "error creating key");
 		return SC_ERROR_INTERNAL;
 	}
 
@@ -825,6 +808,7 @@ static int asepcos_generate_key(sc_profile_t *profile, sc_card_t *card,
 		return SC_ERROR_OUT_OF_MEMORY;
 	memcpy(pubkey->u.rsa.exponent.data, sbuf, 3);
 
+	kinfo->key_reference = tpath.value[1];
 	return SC_SUCCESS;
 }
 
@@ -842,8 +826,9 @@ static struct sc_pkcs15init_operations sc_pkcs15init_asepcos_operations = {
 	asepcos_generate_key,
 	NULL, NULL, 			/* encode private/public key */
 	NULL,				/* finalize_card */
-	NULL, NULL, NULL, NULL, NULL,	/* old style api */
-	NULL 				/* delete_object */
+	NULL, 				/* delete_object */
+	NULL, NULL, NULL, NULL, NULL, 	/* pkcs15init emulation */
+	NULL				/* sanity_check */
 };
 
 struct sc_pkcs15init_operations * sc_pkcs15init_get_asepcos_ops(void)
