@@ -480,7 +480,7 @@ static struct sc_pkcs15init_operations sc_pkcs15init_entersafe_operations = {
 	NULL, NULL,			/* encode private/public key */
 	NULL,	  			/* finalize */
 	NULL, 				/* delete_object */
-	NULL, NULL, NULL, NULL,  /* pkcs15init emulation */
+	NULL, NULL, NULL, NULL, NULL, 	/* pkcs15init emulation */
 	entersafe_sanity_check,
 };
 
