@@ -275,7 +275,8 @@ static struct sc_pkcs15init_operations sc_pkcs15init_miocos_operations = {
 	NULL, NULL,			/* encode private/public key */
 	NULL,				/* finalize_card */
 	NULL, 				/* delete_object */
-	NULL, NULL, NULL, NULL, NULL	/* pkcs15init emulation */
+	NULL, NULL, NULL, NULL, NULL,	/* pkcs15init emulation */
+	NULL				/* sanity_check */
 };
 
 struct sc_pkcs15init_operations *sc_pkcs15init_get_miocos_ops(void)
