@@ -339,7 +339,8 @@ static struct sc_pkcs15init_operations sc_pkcs15init_westcos_operations = {
 	NULL, NULL,				/* encode private/public key */
 	westcos_pkcs15init_finalize_card,	/* finalize_card */
 	NULL,					/* delete_object */
-	NULL, NULL, NULL, NULL, NULL  /* pkcs15init emulation */
+	NULL, NULL, NULL, NULL, NULL,		/* pkcs15init emulation */
+	NULL					/* sanity_check */
 };
 
 struct sc_pkcs15init_operations* sc_pkcs15init_get_westcos_ops(void)
