@@ -38,6 +38,11 @@ struct file_info {
 	struct sc_profile *	base_template;
 	unsigned int		inst_index;
 	sc_path_t		inst_path;
+
+        /* Profile extension dependent on the application ID (sub-profile).
+	 * Sub-profile is loaded when binding to the particular application
+	 * of the multi-application PKCS#15 card. */
+	char *			profile_extension;
 };
 
 /* For now, we assume the PUK always resides
@@ -122,32 +127,23 @@ struct sc_profile {
 };
 
 struct sc_profile *sc_profile_new(void);
-int		sc_profile_load(struct sc_profile *, const char *);
-int		sc_profile_finish(struct sc_profile *);
-void		sc_profile_free(struct sc_profile *);
-int		sc_profile_build_pkcs15(struct sc_profile *);
-void		sc_profile_get_pin_info(struct sc_profile *,
-			int, struct sc_pkcs15_pin_info *);
-int		sc_profile_get_pin_id(struct sc_profile *,
-			unsigned int, int *);
-int		sc_profile_get_file(struct sc_profile *, const char *,
-			struct sc_file **);
-int		sc_profile_get_file_by_path(struct sc_profile *,
-			const struct sc_path *, struct sc_file **);
-int		sc_profile_get_path(struct sc_profile *,
-			const char *, struct sc_path *);
-int		sc_profile_get_file_in(struct sc_profile *,
-			const sc_path_t *, const char *, sc_file_t **);
-int		sc_profile_instantiate_template(struct sc_profile *,
-			const char *, const sc_path_t *,
-			const char *, const sc_pkcs15_id_t *,
-			sc_file_t **);
-int		sc_profile_add_file(struct sc_profile *,
-			const char *, sc_file_t *);
-int		sc_profile_get_file_instance(struct sc_profile *, const char *,
-			int, sc_file_t **);
-int		sc_profile_get_pin_id_by_reference(struct sc_profile *,
-			unsigned, int, struct sc_pkcs15_pin_info *);
+int	sc_profile_load(struct sc_profile *, const char *);
+int	sc_profile_finish(struct sc_profile *, const struct sc_app_info *);
+void	sc_profile_free(struct sc_profile *);
+int	sc_profile_build_pkcs15(struct sc_profile *);
+void	sc_profile_get_pin_info(struct sc_profile *, int, struct sc_pkcs15_pin_info *);
+int	sc_profile_get_pin_id(struct sc_profile *, unsigned int, int *);
+int	sc_profile_get_file(struct sc_profile *, const char *, struct sc_file **);
+int	sc_profile_get_file_by_path(struct sc_profile *, const struct sc_path *, struct sc_file **);
+int	sc_profile_get_path(struct sc_profile *, const char *, struct sc_path *);
+int	sc_profile_get_file_in(struct sc_profile *, const sc_path_t *, const char *, sc_file_t **);
+int	sc_profile_instantiate_template(struct sc_profile *, const char *, const sc_path_t *,
+			const char *, const sc_pkcs15_id_t *, sc_file_t **);
+int	sc_profile_add_file(struct sc_profile *, const char *, sc_file_t *);
+int	sc_profile_get_file_instance(struct sc_profile *, const char *, int, sc_file_t **);
+int	sc_profile_get_pin_id_by_reference(struct sc_profile *, unsigned, int, 
+			struct sc_pkcs15_pin_info *);
+int    sc_profile_get_parent(struct sc_profile *profile, const char *, sc_file_t **);
 
 #ifdef __cplusplus
 }
