@@ -18,9 +18,8 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #ifdef HAVE_UNISTD_H
@@ -30,20 +29,28 @@
 #include <errno.h>
 #include <ctype.h>
 #include <sys/stat.h>
-#include <opensc/opensc.h>
-#include <opensc/cardctl.h>
+
+#include "libopensc/opensc.h"
+#include "libopensc/cardctl.h"
 #include "util.h"
 
+/* type for associations of IDs to names */
+typedef struct _id2str {
+	unsigned int id;
+	const char *str;
+} id2str_t;
+
 static const char *app_name = "opensc-tool";
 
-static int	opt_reader = -1,
-		opt_wait = 0;
+static int	opt_wait = 0;
 static char **	opt_apdus;
+static char	*opt_reader;
 static int	opt_apdu_count = 0;
 static int	verbose = 0;
 
 enum {
 	OPT_SERIAL = 0x100,
+	OPT_LIST_ALG
 };
 
 static const struct option options[] = {
@@ -55,11 +62,11 @@ static const struct option options[] = {
 	{ "set-conf-entry",	1, NULL,		'S' },
 	{ "list-readers",	0, NULL, 		'l' },
 	{ "list-drivers",	0, NULL,		'D' },
-	{ "list-rdrivers",	0, NULL,		'R' },
 	{ "list-files",		0, NULL,		'f' },
 	{ "send-apdu",		1, NULL,		's' },
 	{ "reader",		1, NULL,		'r' },
 	{ "card-driver",	1, NULL,		'c' },
+	{ "list-algorithms",    0, NULL,	OPT_LIST_ALG }, 
 	{ "wait",		0, NULL,		'w' },
 	{ "verbose",		0, NULL,		'v' },
 	{ NULL, 0, NULL, 0 }
@@ -72,13 +79,13 @@ static const char *option_help[] = {
 	"Identify the card and print its name",
 	"Get configuration key, format: section:name:key",
 	"Set configuration key, format: section:name:key:value",
-	"Lists all configured readers",
+	"Lists readers",
 	"Lists all installed card drivers",
-	"Lists all installed reader drivers",
 	"Recursively lists files stored on card",
 	"Sends an APDU in format AA:BB:CC:DD:EE:FF...",
 	"Uses reader number <arg> [0]",
 	"Forces the use of driver <arg> [auto-detect]",
+	"Lists algorithms supported by card",
 	"Wait for a card to be inserted",
 	"Verbose operation. Use several times to enable debug output.",
 };
@@ -104,6 +111,15 @@ static int opensc_info(void)
 #endif
 		__VERSION__
 	);
+#elif defined(__SUNPRO_C)
+	printf (
+		"[Sun C %x.%x]\n",
+#if __SUNPRO_C > 0x590
+		(__SUNPRO_C >> 12), (__SUNPRO_C >> 4) & 0xFF
+#else
+		(__SUNPRO_C >>  8), (__SUNPRO_C >> 4) & 0xF
+#endif
+	);
 #elif defined(_MSC_VER)
 	printf ("[Microsoft %d]\n", _MSC_VER);
 #else
@@ -212,7 +228,7 @@ static int opensc_set_conf_entry(const char *config)
 			scconf_list *list;
 
 			if ((item->type != SCCONF_ITEM_TYPE_VALUE)
-			    || (strcmp(item->key, key) != 0))
+			   || (strcmp(item->key, key) != 0))
 				continue;
 			list = item->value.list;
 			scconf_list_destroy(list);
@@ -249,28 +265,31 @@ static int list_readers(void)
 		printf("No smart card readers found.\n");
 		return 0;
 	}
-	printf("Readers known about:\n");
-	printf("Nr.    Driver     Name\n");
+	printf("# Detected readers (%s)\n", ctx->reader_driver->short_name);
+	printf("Nr.  Card  Features  Name\n");
 	for (i = 0; i < rcount; i++) {
-		sc_reader_t *screader = sc_ctx_get_reader(ctx, i);
-		printf("%-7d%-11s%s\n", i, screader->driver->short_name,
-		       screader->name);
-	}
-	return 0;
-}
+		sc_reader_t *reader = sc_ctx_get_reader(ctx, i);
+		int state = sc_detect_card_presence(reader);
+		printf("%-5d%-6s%-10s%s\n", i, state & SC_READER_CARD_PRESENT ? "Yes":"No",
+		      reader->capabilities & SC_READER_CAP_PIN_PAD ? "PIN pad":"",
+		      reader->name);
+		if (state & SC_READER_CARD_PRESENT && verbose) {
+			struct sc_card *card;
+			int r;
+			char tmp[SC_MAX_ATR_SIZE*3];
+			sc_bin_to_hex(reader->atr.value, reader->atr.len, tmp, sizeof(tmp) - 1, ':');
 
-static int list_reader_drivers(void)
-{
-	int i;
-	
-	if (ctx->reader_drivers[0] == NULL) {
-		printf("No reader drivers installed!\n");
-		return 0;
-	}
-	printf("Configured reader drivers:\n");
-	for (i = 0; ctx->reader_drivers[i] != NULL; i++) {
-		printf("  %-16s %s\n", ctx->reader_drivers[i]->short_name,
-		       ctx->reader_drivers[i]->name);
+			if (state & SC_READER_CARD_EXCLUSIVE)
+				printf("     %s [EXCLUSIVE]\n", tmp);
+			else {		
+				if ((r = sc_connect_card(reader, &card)) != SC_SUCCESS) {
+					fprintf(stderr, "     failed: %s\n", sc_strerror(r));
+				} else {
+					printf("     %s %s %s\n", tmp, card->name, state & SC_READER_CARD_INUSE ? "[IN USE]" : "");
+					sc_disconnect_card(card);
+				}
+			}
+		}
 	}
 	return 0;
 }
@@ -286,7 +305,7 @@ static int list_drivers(void)
 	printf("Configured card drivers:\n");
 	for (i = 0; ctx->card_drivers[i] != NULL; i++) {
 		printf("  %-16s %s\n", ctx->card_drivers[i]->short_name,
-		       ctx->card_drivers[i]->name);
+		      ctx->card_drivers[i]->name);
 	}
 	return 0;
 }
@@ -296,14 +315,7 @@ static int print_file(sc_card_t *in_card, const sc_file_t *file,
 {
 	int r;
 	const char *tmps;
-	const char *ac_ops_df[] = {
-		"select", "lock", "delete", "create", "rehab", "inval",
-		"list"
-	};
-	const char *ac_ops_ef[] = {
-		"read", "update", "erase", "write", "rehab", "inval"
-	};
-	
+
 	for (r = 0; r < depth; r++)
 		printf("  ");
 	printf("%s ", sc_print_path(path));
@@ -328,33 +340,67 @@ static int print_file(sc_card_t *in_card, const sc_file_t *file,
 	}
 	printf("type: %-3s, ", tmps);
 	if (file->type != SC_FILE_TYPE_DF) {
-		const char *structs[] = {
-			"unknown", "transpnt", "linrfix", "linrfix(TLV)",
-			"linvar", "linvar(TLV)", "lincyc", "lincyc(TLV)"
+		const id2str_t ef_type_name[] = {
+			{ SC_FILE_EF_TRANSPARENT,         "transpnt"     },
+			{ SC_FILE_EF_LINEAR_FIXED,        "linrfix"      },
+			{ SC_FILE_EF_LINEAR_FIXED_TLV,    "linrfix(TLV)" },
+			{ SC_FILE_EF_LINEAR_VARIABLE,     "linvar"       },
+			{ SC_FILE_EF_LINEAR_VARIABLE_TLV, "linvar(TLV)"  },
+			{ SC_FILE_EF_CYCLIC,              "lincyc"       },
+			{ SC_FILE_EF_CYCLIC_TLV,          "lincyc(TLV)"  },
+			{ 0, NULL }
 		};
-		int ef_type = file->ef_structure;
-		if (ef_type < 0 || ef_type > 7)
-			ef_type = 0;	/* invalid or unknow ef type */
-		printf("ef structure: %s, ", structs[ef_type]);
+		const char *ef_type = "unknown";
+
+		for (r = 0; ef_type_name[r].str != NULL; r++)
+			if (file->ef_structure == ef_type_name[r].id)
+				ef_type = ef_type_name[r].str;
+
+		printf("ef structure: %s, ", ef_type);
 	}
 	printf("size: %lu\n", (unsigned long) file->size);
 	for (r = 0; r < depth; r++)
 		printf("  ");
-	if (file->type == SC_FILE_TYPE_DF)
-		for (r = 0; r < (int) (sizeof(ac_ops_df)/sizeof(ac_ops_df[0])); r++)
-			printf("%s[%s] ", ac_ops_df[r], util_acl_to_str(sc_file_get_acl_entry(file, r)));
-	else
-		for (r = 0; r < (int) (sizeof(ac_ops_ef)/sizeof(ac_ops_ef[0])); r++)
-			printf("%s[%s] ", ac_ops_ef[r], util_acl_to_str(sc_file_get_acl_entry(file, r)));
+	if (file->type == SC_FILE_TYPE_DF) {
+		const id2str_t ac_ops_df[] = {
+			{ SC_AC_OP_SELECT,       "select" },
+			{ SC_AC_OP_LOCK,         "lock"   },
+			{ SC_AC_OP_DELETE,       "delete" },
+			{ SC_AC_OP_CREATE,       "create" },
+			{ SC_AC_OP_REHABILITATE, "rehab"  },
+			{ SC_AC_OP_INVALIDATE,   "inval"  },
+			{ SC_AC_OP_LIST_FILES,   "list"   },
+			{ 0, NULL }
+		};
+
+		for (r = 0; ac_ops_df[r].str != NULL; r++)
+			printf("%s[%s] ", ac_ops_df[r].str,
+					 util_acl_to_str(sc_file_get_acl_entry(file, ac_ops_df[r].id)));
+	}
+	else {
+		const id2str_t ac_ops_ef[] = {
+			{ SC_AC_OP_READ,         "read"   },
+			{ SC_AC_OP_UPDATE,       "update" },
+			{ SC_AC_OP_ERASE,        "erase"  },
+			{ SC_AC_OP_WRITE,        "write"  },
+			{ SC_AC_OP_REHABILITATE, "rehab"  },
+			{ SC_AC_OP_INVALIDATE,   "inval"  },
+			{ 0, NULL }
+		};
+
+		for (r = 0; ac_ops_ef[r].str != NULL; r++)
+			printf("%s[%s] ", ac_ops_ef[r].str,
+					util_acl_to_str(sc_file_get_acl_entry(file, ac_ops_ef[r].id)));
+	}
 
 	if (file->sec_attr_len) {
 		printf("sec: ");
 		/* Octets are as follows:
-		 *   DF: select, lock, delete, create, rehab, inval
-		 *   EF: read, update, write, erase, rehab, inval
-		 * 4 MSB's of the octet mean:			 
-		 *  0 = ALW, 1 = PIN1, 2 = PIN2, 4 = SYS,
-		 * 15 = NEV */
+		*   DF: select, lock, delete, create, rehab, inval
+		*   EF: read, update, write, erase, rehab, inval
+		* 4 MSB's of the octet mean:			
+		*  0 = ALW, 1 = PIN1, 2 = PIN2, 4 = SYS,
+		* 15 = NEV */
 		util_hex_dump(stdout, file->sec_attr, file->sec_attr_len, ":");
 	}
 	if (file->prop_attr_len) {
@@ -372,7 +418,7 @@ static int print_file(sc_card_t *in_card, const sc_file_t *file,
 	if (file->ef_structure == SC_FILE_EF_TRANSPARENT) {
 		unsigned char *buf;
 		
-		if (!(buf = (unsigned char *) malloc(file->size))) {
+		if (!(buf = malloc(file->size))) {
 			fprintf(stderr, "out of memory");
 			return 1;
 		}
@@ -423,6 +469,7 @@ static int enum_dir(sc_path_t path, int depth)
 		for (i = 0; i < r/2; i++) {
 			sc_path_t tmppath;
 
+			memset(&tmppath, 0, sizeof(tmppath));
 			memcpy(&tmppath, &path, sizeof(path));
 			memcpy(tmppath.value + tmppath.len, files + 2*i, 2);
 			tmppath.len += 2;
@@ -445,62 +492,24 @@ static int list_files(void)
 static int send_apdu(void)
 {
 	sc_apdu_t apdu;
-	u8 buf[SC_MAX_APDU_BUFFER_SIZE], sbuf[SC_MAX_APDU_BUFFER_SIZE],
-	   rbuf[SC_MAX_APDU_BUFFER_SIZE], *p;
-	size_t len, len0, r;
+	u8 buf[SC_MAX_APDU_BUFFER_SIZE],
+	  rbuf[SC_MAX_APDU_BUFFER_SIZE];
+	size_t len0, r;
 	int c;
 
 	for (c = 0; c < opt_apdu_count; c++) {
 		len0 = sizeof(buf);
 		sc_hex_to_bin(opt_apdus[c], buf, &len0);
-		if (len0 < 4) {
-			fprintf(stderr, "APDU too short (must be at least 4 bytes).\n");
+
+		r = sc_bytes2apdu(card->ctx, buf, len0, &apdu);
+		if (r) {
+			fprintf(stderr, "Invalid APDU: %s\n", sc_strerror(r));
 			return 2;
 		}
-		len = len0;
-		p = buf;
-		memset(&apdu, 0, sizeof(apdu));
-		apdu.cla = *p++;
-		apdu.ins = *p++;
-		apdu.p1 = *p++;
-		apdu.p2 = *p++;
+
 		apdu.resp = rbuf;
 		apdu.resplen = sizeof(rbuf);
-		len -= 4;
-		if (len > 1) {
-			apdu.lc = *p++;
-			len--;
-			memcpy(sbuf, p, apdu.lc);
-			apdu.data = sbuf;
-			apdu.datalen = apdu.lc;
-			if (len < apdu.lc) {
-				fprintf(stderr, "APDU too short (need %lu bytes).\n",
-					(unsigned long) apdu.lc-len);
-				return 2;
-			}
-			len -= apdu.lc;
-			p   += apdu.lc;
-			if (len) {
-				apdu.le = *p++;
-				if (apdu.le == 0)
-					apdu.le = 256;
-				len--;
-				apdu.cse = SC_APDU_CASE_4_SHORT;
-			} else
-				apdu.cse = SC_APDU_CASE_3_SHORT;
-			if (len) {
-				fprintf(stderr, "APDU too long (%lu bytes extra).\n",
-					(unsigned long) len);
-				return 2;
-			}
-		} else if (len == 1) {
-			apdu.le = *p++;
-			if (apdu.le == 0)
-				apdu.le = 256;
-			len--;
-			apdu.cse = SC_APDU_CASE_2_SHORT;
-		} else
-			apdu.cse = SC_APDU_CASE_1;
+
 		printf("Sending: ");
 		for (r = 0; r < len0; r++)
 			printf("%02X ", buf[r]);
@@ -511,7 +520,7 @@ static int send_apdu(void)
 			return 1;
 		}
 		printf("Received (SW1=0x%02X, SW2=0x%02X)%s\n", apdu.sw1, apdu.sw2,
-		       apdu.resplen ? ":" : "");
+		      apdu.resplen ? ":" : "");
 		if (apdu.resplen)
 			util_hex_dump_asc(stdout, apdu.resp, apdu.resplen, -1);
 	}
@@ -530,6 +539,106 @@ static void print_serial(sc_card_t *in_card)
 		util_hex_dump_asc(stdout, serial.value, serial.len, -1);
 }
 
+static int list_algorithms(void) 
+{
+	int i; 
+	const char *aname = "unknown";
+
+	const id2str_t alg_type_names[] = {
+		{ SC_ALGORITHM_RSA,       "rsa"       },
+		{ SC_ALGORITHM_DSA,       "dsa"       },
+		{ SC_ALGORITHM_EC,        "ec"        },
+		{ SC_ALGORITHM_GOSTR3410, "gostr3410" },
+		{ SC_ALGORITHM_DES,       "des"       },
+		{ SC_ALGORITHM_3DES,      "3des"      },
+		{ SC_ALGORITHM_GOST,      "gost"      },
+		{ SC_ALGORITHM_MD5,       "md5"       },
+		{ SC_ALGORITHM_SHA1,      "sha1"      },
+		{ SC_ALGORITHM_GOSTR3411, "gostr3411" },
+		{ SC_ALGORITHM_PBKDF2,    "pbkdf2"    },
+		{ SC_ALGORITHM_PBES2,     "pbes2"     },
+		{ 0, NULL }
+	};
+	const id2str_t alg_flag_names[] = {
+		{ SC_ALGORITHM_ONBOARD_KEY_GEN, "onboard key generation" },
+		{ SC_ALGORITHM_NEED_USAGE,      "needs usage"            },
+		{ 0, NULL }
+	};
+	const id2str_t rsa_flag_names[] = {
+		{ SC_ALGORITHM_RSA_PAD_PKCS1,      "pkcs1"     },
+		{ SC_ALGORITHM_RSA_PAD_ANSI,       "ansi"      },
+		{ SC_ALGORITHM_RSA_PAD_ISO9796,    "iso9796"   },
+		{ SC_ALGORITHM_RSA_HASH_SHA1,      "sha1"      },
+		{ SC_ALGORITHM_RSA_HASH_MD5,       "MD5"       },
+		{ SC_ALGORITHM_RSA_HASH_MD5_SHA1,  "md5-sha1"  },
+		{ SC_ALGORITHM_RSA_HASH_RIPEMD160, "ripemd160" },
+		{ SC_ALGORITHM_RSA_HASH_SHA256,    "sha256"    },
+		{ SC_ALGORITHM_RSA_HASH_SHA384,    "sha384"    },
+		{ SC_ALGORITHM_RSA_HASH_SHA512,    "sha512"    },
+		{ SC_ALGORITHM_RSA_HASH_SHA224,    "sha224"    },
+		{ 0, NULL }
+	};
+
+	if (verbose)
+		printf("Card supports %d algorithm(s)\n\n",card->algorithm_count); 
+  
+	for (i=0; i < card->algorithm_count; i++) { 
+		int j;
+
+		/* find algorithm name */
+		for (j = 0; alg_type_names[j].str != NULL; j++) {
+			if (card->algorithms[i].algorithm == alg_type_names[j].id) {
+				aname = alg_type_names[j].str;
+				break;
+			}
+		}
+
+		printf("Algorithm: %s\n", aname); 
+		printf("Key length: %d\n", card->algorithms[i].key_length); 
+		printf("Flags:"); 
+
+		/* print general flags */
+		for (j = 0; alg_flag_names[j].str != NULL; j++)
+			if (card->algorithms[i].flags & alg_flag_names[j].id)
+				printf(" %s", alg_flag_names[j].str);
+
+		/* print RSA spcific flags */
+		if ( card->algorithms[i].algorithm == SC_ALGORITHM_RSA) { 
+			int padding = card->algorithms[i].flags 
+					& SC_ALGORITHM_RSA_PADS; 
+			int hashes =  card->algorithms[i].flags 
+					& SC_ALGORITHM_RSA_HASHES; 
+					 
+			/* print RSA padding flags */
+			printf(" padding ("); 
+			for (j = 0; rsa_flag_names[j].str != NULL; j++)
+				if (padding & rsa_flag_names[j].id)
+					printf(" %s", rsa_flag_names[j].str);
+			if (padding == SC_ALGORITHM_RSA_PAD_NONE)  
+				printf(" none"); 
+			printf(" ) "); 
+			/* print RSA hash flags */
+			printf("hashes ("); 
+			for (j = 0; rsa_flag_names[j].str != NULL; j++)
+				if (hashes & rsa_flag_names[j].id)
+					printf(" %s", rsa_flag_names[j].str);
+			if (hashes == SC_ALGORITHM_RSA_HASH_NONE)
+				printf(" none"); 
+			printf(" )"); 
+		} 
+		printf("\n"); 
+		if (card->algorithms[i].algorithm == SC_ALGORITHM_RSA  
+			&& card->algorithms[i].u._rsa.exponent) { 
+			printf("RSA public exponent: %lu\n", (unsigned long) 
+				card->algorithms[i].u._rsa.exponent);
+		} 
+  
+		if (i < card->algorithm_count) 
+			printf("\n"); 
+	} 
+	return 0; 
+} 
+
 int main(int argc, char * const argv[])
 {
 	int err = 0, r, c, long_optind = 0;
@@ -538,12 +647,12 @@ int main(int argc, char * const argv[])
 	int do_set_conf_entry = 0;
 	int do_list_readers = 0;
 	int do_list_drivers = 0;
-	int do_list_rdrivers = 0;
 	int do_list_files = 0;
 	int do_send_apdu = 0;
 	int do_print_atr = 0;
 	int do_print_serial = 0;
 	int do_print_name = 0;
+	int do_list_algorithms = 0;
 	int action_count = 0;
 	const char *opt_driver = NULL;
 	const char *opt_conf_entry = NULL;
@@ -553,7 +662,7 @@ int main(int argc, char * const argv[])
 	setbuf(stdout, NULL);
 
 	while (1) {
-		c = getopt_long(argc, argv, "inlG:S:fr:vs:DRc:aw", options, &long_optind);
+		c = getopt_long(argc, argv, "inlG:S:fr:vs:Dc:aw", options, &long_optind);
 		if (c == -1)
 			break;
 		if (c == '?')
@@ -581,10 +690,6 @@ int main(int argc, char * const argv[])
 			do_list_drivers = 1;
 			action_count++;
 			break;
-		case 'R':
-			do_list_rdrivers = 1;
-			action_count++;
-			break;
 		case 'f':
 			do_list_files = 1;
 			action_count++;
@@ -607,7 +712,7 @@ int main(int argc, char * const argv[])
 			action_count++;
 			break;
 		case 'r':
-			opt_reader = atoi(optarg);
+			opt_reader = optarg;
 			break;
 		case 'v':
 			verbose++;
@@ -622,6 +727,10 @@ int main(int argc, char * const argv[])
 			do_print_serial = 1;
 			action_count++;
 			break;
+		case OPT_LIST_ALG:
+			do_list_algorithms = 1; 
+			action_count++; 
+			break;
 		}
 	}
 	if (action_count == 0)
@@ -641,8 +750,12 @@ int main(int argc, char * const argv[])
 		fprintf(stderr, "Failed to establish context: %s\n", sc_strerror(r));
 		return 1;
 	}
-	if (verbose > 1)
-		ctx->debug = verbose-1;
+
+	if (verbose > 1) {
+		ctx->debug = verbose;
+		sc_ctx_log_to_file(ctx, "stderr");
+	}
+
 	if (do_get_conf_entry) {
 		if ((err = opensc_get_conf_entry (opt_conf_entry)))
 			goto end;
@@ -653,11 +766,6 @@ int main(int argc, char * const argv[])
 			goto end;
 		action_count--;
 	}
-	if (do_list_rdrivers) {
-		if ((err = list_reader_drivers()))
-			goto end;
-		action_count--;
-	}
 	if (do_list_readers) {
 		if ((err = list_readers()))
 			goto end;
@@ -680,17 +788,17 @@ int main(int argc, char * const argv[])
 		}
 	}
 
-	err = util_connect_card(ctx, &card, opt_reader, 0, opt_wait, verbose);
+	err = util_connect_card(ctx, &card, opt_reader, opt_wait, verbose);
 	if (err)
 		goto end;
 
 	if (do_print_atr) {
 		if (verbose) {
 			printf("Card ATR:\n");
-			util_hex_dump_asc(stdout, card->atr, card->atr_len, -1);		
+			util_hex_dump_asc(stdout, card->atr.value, card->atr.len, -1);		
 		} else {
 			char tmp[SC_MAX_ATR_SIZE*3];
-			sc_bin_to_hex(card->atr, card->atr_len, tmp, sizeof(tmp) - 1, ':');
+			sc_bin_to_hex(card->atr.value, card->atr.len, tmp, sizeof(tmp) - 1, ':');
 			fprintf(stdout,"%s\n",tmp);
 		}
 		action_count--;
@@ -718,10 +826,16 @@ int main(int argc, char * const argv[])
 			goto end;
 		action_count--;
 	}
+
+	if (do_list_algorithms) { 
+		if ((err = list_algorithms())) 
+			goto end;
+		action_count--; 
+	} 
 end:
 	if (card) {
 		sc_unlock(card);
-		sc_disconnect_card(card, 0);
+		sc_disconnect_card(card);
 	}
 	if (ctx)
 		sc_release_context(ctx);
