@@ -133,6 +133,7 @@ enum {
 	OPT_PUK_LABEL,
 	OPT_VERIFY_PIN,
 	OPT_SANITY_CHECK,
+	OPT_BIND_TO_AID,
 
 	OPT_PIN1     = 0x10000,	/* don't touch these values */
 	OPT_PUK1     = 0x10001,
@@ -174,6 +175,7 @@ const struct option	options[] = {
 	{ "cert-label",		required_argument, NULL,	OPT_CERT_LABEL },
 	{ "application-name",	required_argument, NULL,	OPT_APPLICATION_NAME },
 	{ "application-id",	required_argument, NULL,	OPT_APPLICATION_ID },
+	{ "aid",		required_argument, NULL,        OPT_BIND_TO_AID },
 	{ "output-file",	required_argument, NULL,	'o' },
 	{ "format",		required_argument, NULL,	'f' },
 	{ "passphrase",		required_argument, NULL,	OPT_PASSPHRASE },
@@ -218,7 +220,7 @@ static const char *		option_help[] = {
 	"Specify unblock PIN",
 	"Specify security officer (SO) PIN",
 	"Specify unblock PIN for SO PIN",
-	"Do not install a SO PIN, and dont prompt for it",
+	"Do not install a SO PIN, and do not prompt for it",
 	"Specify the serial number of the card",
 	"Specify ID of PIN to use/create",
 	"Specify ID of PUK to use/create",
@@ -230,6 +232,7 @@ static const char *		option_help[] = {
 	"Specify user cert label (use with --store-private-key)",
 	"Specify application name of data object (use with --store-data-object)",
 	"Specify application id of data object (use with --store-data-object)",
+	"Specify AID of the on-card PKCS#15 application to be binded to (in hexadecimal form)",
 	"Output public portion of generated key to file",
 	"Specify key/cert file format: PEM (=default), DER or PKCS12",
 	"Specify passphrase for unlocking secret key",
@@ -337,6 +340,7 @@ static char *			opt_newkey = NULL;
 static char *			opt_outkey = NULL;
 static char *			opt_application_id = NULL;
 static char *			opt_application_name = NULL;
+static char *			opt_bind_to_aid = NULL;
 static char *			opt_puk_authid = NULL;
 static unsigned int		opt_x509_usage = 0;
 static unsigned int		opt_delete_flags = 0;
@@ -450,11 +454,28 @@ main(int argc, char **argv)
 		 && action != ACTION_ASSERT_PRISTINE
 		 && p15card == NULL) {
 			/* Read the PKCS15 structure from the card */
-			r = sc_pkcs15_bind(card, &p15card);
+			if (opt_bind_to_aid)   {
+				struct sc_aid aid;
+
+				aid.len = sizeof(aid.value);
+				if (sc_hex_to_bin(opt_bind_to_aid, aid.value, &aid.len))   {
+					fprintf(stderr, "Invalid AID value: '%s'\n", opt_bind_to_aid);
+					return 1;
+				}
+
+				r = sc_pkcs15init_finalize_profile(card, profile, &aid);
+				if (r < 0)   {
+					fprintf(stderr, "Finalize profile error %s\n", sc_strerror(r));
+					break;
+				}
+
+				r = sc_pkcs15_bind(card, &aid, &p15card);
+			}
+			else   {	
+				r = sc_pkcs15_bind(card, NULL, &p15card);
+			}
 			if (r) {
-				fprintf(stderr,
-					"PKCS#15 binding failed: %s\n",
-					sc_strerror(r));
+				fprintf(stderr, "PKCS#15 binding failed: %s\n", sc_strerror(r));
 				break;
 			}
 
@@ -568,7 +589,7 @@ open_reader_and_card(char *reader)
 
 	if (verbose > 1) {
 		ctx->debug = verbose;
-		ctx->debug_file = stderr;
+		sc_ctx_log_to_file(ctx, "stderr");
 	}
 
 	if (util_connect_card(ctx, &card, reader, opt_wait, verbose))
@@ -627,10 +648,23 @@ do_erase(sc_card_t *in_card, struct sc_profile *profile)
 
 	p15card = sc_pkcs15_card_new();
 	p15card->card = in_card;
-	p15card->tokeninfo->label = strdup("Dummy PKCS#15 object");
 
 	ignore_cmdline_pins++;
-	r = sc_pkcs15init_erase_card(p15card, profile);
+	if (opt_bind_to_aid)   {
+		struct sc_aid aid;
+
+		aid.len = sizeof(aid.value);
+		if (sc_hex_to_bin(opt_bind_to_aid, aid.value, &aid.len))   {
+			fprintf(stderr, "Invalid AID value: '%s'\n", opt_bind_to_aid);
+			return 1;
+									                
+		}
+
+		r = sc_pkcs15init_erase_card(p15card, profile, &aid);
+	}
+	else   {
+		r = sc_pkcs15init_erase_card(p15card, profile, NULL);
+	}
 	ignore_cmdline_pins--;
 
 	sc_pkcs15_card_free(p15card);
@@ -821,7 +855,7 @@ do_store_private_key(struct sc_profile *profile)
 
 	if ((r = do_convert_private_key(&args.key, pkey)) < 0)
 		return r;
-	init_gost_params(&args.gost_params, pkey);
+	init_gost_params(&args.params.gost, pkey);
 
 	if (ncerts) {
 		unsigned int	usage;
@@ -968,11 +1002,10 @@ do_store_public_key(struct sc_profile *profile, EVP_PKEY *pkey)
 	if (r >= 0) {
 		r = do_convert_public_key(&args.key, pkey);
 		if (r >= 0)
-			init_gost_params(&args.gost_params, pkey);
+			init_gost_params(&args.params.gost, pkey);
 	}
 	if (r >= 0)
-		r = sc_pkcs15init_store_public_key(p15card, profile,
-					&args, &dummy);
+		r = sc_pkcs15init_store_public_key(p15card, profile, &args, &dummy);
 
 	return r;
 }
@@ -1421,23 +1454,32 @@ do_generate_key(struct sc_profile *profile, const char *spec)
 		keygen_args.prkey_args.key.algorithm = SC_ALGORITHM_GOSTR3410;
 		keybits = SC_PKCS15_GOSTR3410_KEYSIZE;
 		/* FIXME: now only SC_PKCS15_PARAMSET_GOSTR3410_A */
-		keygen_args.prkey_args.gost_params.gostr3410 =
-			SC_PKCS15_PARAMSET_GOSTR3410_A;
+		keygen_args.prkey_args.params.gost.gostr3410 = SC_PKCS15_PARAMSET_GOSTR3410_A;
 		spec += strlen("gost2001");
+	} else if (!strncasecmp(spec, "ec", 2)) {
+		keygen_args.prkey_args.key.algorithm = SC_ALGORITHM_EC;
+		spec += 2;
 	} else {
 		util_error("Unknown algorithm \"%s\"", spec);
 		return SC_ERROR_INVALID_ARGUMENTS;
 	}
 
-	if (*spec == '/' || *spec == '-')
+	if (*spec == '/' || *spec == '-' || *spec == ':')
 		spec++;
-	if (*spec) {
-		char	*end;
 
-		keybits = strtoul(spec, &end, 10);
-		if (*end) {
-			util_error("Invalid number of key bits \"%s\"", spec);
-			return SC_ERROR_INVALID_ARGUMENTS;
+	if (*spec)   {
+		if (isalpha(*spec) && keygen_args.prkey_args.key.algorithm == SC_ALGORITHM_EC)   {
+			keygen_args.prkey_args.params.ec.named_curve = strdup(spec);
+			keybits = 0;
+		}
+		else {
+			char	*end;
+
+			keybits = strtoul(spec, &end, 10);
+			if (*end) {
+				util_error("Invalid number of key bits \"%s\"", spec);
+				return SC_ERROR_INVALID_ARGUMENTS;
+			}
 		}
 	}
 	r = sc_pkcs15init_generate_key(p15card, profile, &keygen_args, keybits, NULL);
@@ -2511,6 +2553,9 @@ handle_option(const struct option *opt)
 	case OPT_APPLICATION_ID:
 		opt_application_id = optarg;
 		break;
+	case OPT_BIND_TO_AID:
+		opt_bind_to_aid = optarg;
+		break;
 	case OPT_PUK_ID:
 		opt_puk_authid = optarg;
 		break;
@@ -2799,7 +2844,7 @@ static int verify_pin(struct sc_pkcs15_card *p15card, char *auth_id_str)
 
 	if (!auth_id_str)   {
 	        struct sc_pkcs15_object *objs[32];
-        	int r, ii;
+        	int ii;
 		
 		r = sc_pkcs15_get_objects(p15card, SC_PKCS15_TYPE_AUTH_PIN, objs, 32);
 		if (r < 0) {
